# Copyright 2021-2025 Zenauth Ltd.
# SPDX-License-Identifier: Apache-2.0
import json
import os
import ssl
import uuid
from functools import wraps
from typing import Any, Dict, List, Optional, Tuple, Union

import grpc

from cerbos.engine.v1 import engine_pb2
from cerbos.policy.v1 import policy_pb2
from cerbos.request.v1 import request_pb2
from cerbos.response.v1 import response_pb2
from cerbos.schema.v1 import schema_pb2
from cerbos.sdk.grpc.utils import AdminCredentials, get_resource, is_allowed
from cerbos.sdk.model import CerbosTLSError, CerbosTypeError
from cerbos.svc.v1 import svc_pb2_grpc

_PLAYGROUND_INSTANCE_KEY = "playground-instance"
_default_paths = ssl.get_default_verify_paths()
TLSVerify = Union[str, bool]
# TODO(saml) type errors generated from passing incorrect types to proto generated code currently
# aren't great, e.g. passing the incorrect Principal type to CheckResourcesRequest results in:
#     "Message must be initialized with a dict: cerbos.request.v1.CheckResourcesRequest".
# Investigate more useful TypeError returns.


def handle_errors(method):
    @wraps(method)
    def wrapper(*args, **kwargs):
        try:
            return method(*args, **kwargs)
        except TypeError as e:
            raise CerbosTypeError(str(e))

    return wrapper


def get_cert(c: TLSVerify) -> Union[bytes, None]:
    try:
        if isinstance(c, str):
            with open(c, "rb") as f:
                return f.read()
        elif isinstance(c, bool):
            # Attempt to retrieve the cert from the location specified at `SSL_CERT_FILE`
            # or the default location if not specified.
            filename = _default_paths.cafile
            if cf := os.getenv("SSL_CERT_FILE"):
                filename = cf
            with open(filename, "rb") as f:
                return f.read()
    except IOError:
        raise CerbosTLSError(f"Error reading certificate from file: {c}")
    except Exception:
        raise CerbosTLSError("Error retrieving certificate")
    raise TypeError("TLSVerify should be a string or boolean")


class PlaygroundInstanceCredentials(grpc.AuthMetadataPlugin):
    def __init__(self, playground_instance: str):
        self._playground_instance = playground_instance

    def __call__(
        self,
        context: grpc.AuthMetadataContext,
        callback: grpc.AuthMetadataPluginCallback,
    ) -> None:
        callback(((_PLAYGROUND_INSTANCE_KEY, self._playground_instance),), None)


class ClientBase:
    _channel: grpc.Channel

    def __init__(
        self,
        host: str,
        creds: Optional[grpc.ChannelCredentials],
        methods: Optional[List[Dict[str, str]]] = None,
        tls_verify: TLSVerify = False,
        timeout_secs: Union[float, None] = None,
        request_retries: int = 0,
        wait_for_ready: bool = False,
        channel_options: Union[Dict[str, Any], None] = None,
    ):
        if timeout_secs and (not isinstance(timeout_secs, (int, float))):
            raise TypeError("timeout_secs must be a number type")
        if request_retries and (not isinstance(request_retries, (int, float))):
            raise TypeError(
                "request_retries must be a number type. anything below 2 is treated as 0 (disabled)"
            )
        if request_retries < 2:
            request_retries = 0
        method_config: Dict[str, Any] = {}
        if methods:
            method_config["name"] = methods
        if timeout_secs:
            method_config["timeout"] = f"{timeout_secs}s"
        if request_retries:
            method_config["retryPolicy"] = {
                "maxAttempts": request_retries,
                "initialBackoff": "1s",
                "maxBackoff": "10s",
                "backoffMultiplier": 2,
                "retryableStatusCodes": ["UNAVAILABLE"],
            }
        if wait_for_ready:
            method_config["waitForReady"] = wait_for_ready
        options = {"grpc.service_config": json.dumps({"methodConfig": [method_config]})}
        if channel_options:
            options = {**options, **channel_options}
        opts = [(k, v) for k, v in options.items()]
        if tls_verify and creds:
            self._channel = grpc.secure_channel(host, credentials=creds, options=opts)
        else:
            self._channel = grpc.insecure_channel(host, options=opts)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def close(self):
        self._channel.close()


class CerbosClient(ClientBase):
    """Client for accessing the Cerbos API

    Args:
        host (str): The full address of the Cerbos API server (PDP)
        tls_verify (bool|str): If a path is passed it is used as the CA certificate. If True, we look for the path specified in `SSL_CERT_FILE` or at the default OS location. If False, server certificate verification is disabled
        playground_instance (str): Optional Cerbos Playground ID if testing against a Playground playground_instance
        timeout_secs (float): Optional request timeout in seconds (no timeout by default)
        request_retries (int): Optional maximum number of retries, including the original attempt. Anything below 2 will be treated as 0 (disabled)
        wait_for_ready (bool): Boolean specifying whether RPCs should wait until the connection is ready. Defaults to False
        channel_options (Dict[str, Any]): Optional gRPC channel options to pass on channel creation. The values need to match the expected types: https://github.com/grpc/grpc/blob/7536d8a849c0096e4c968e7730306872bb5ec674/include/grpc/impl/grpc_types.h

    Example:
        with AsyncCerbosClient("localhost:3593") as cerbos:
            if cerbos.is_allowed(
                "view",
                Principal(id="john", roles={"employee"}, attr={"geography": struct_pb2.Value(string_value="GB")}),
                Resource(id="XX125", "kind"="leave_request", attr={"geography": struct_pb2.Value(string_value="GB")})
                ):
                do_thing()
    """

    _client: svc_pb2_grpc.CerbosServiceStub

    def __init__(
        self,
        host: str,
        tls_verify: TLSVerify = False,
        playground_instance: str = "",
        timeout_secs: Union[float, None] = None,
        request_retries: int = 0,
        wait_for_ready: bool = False,
        channel_options: Union[Dict[str, Any], None] = None,
    ):
        creds: Optional[grpc.ChannelCredentials] = None
        if tls_verify:
            cert = get_cert(tls_verify)
            creds = grpc.ssl_channel_credentials(cert)
        if playground_instance:
            # insecure creds required for playground without TLS
            if not creds:
                creds = grpc.ssl_channel_credentials()
            call_credentials = grpc.metadata_call_credentials(
                PlaygroundInstanceCredentials(playground_instance)
            )
            creds = grpc.composite_channel_credentials(creds, call_credentials)
        methods = [
            {"service": "svc.CerbosService", "method": "CheckResources"},
            {"service": "svc.CerbosService", "method": "PlanResources"},
        ]
        super().__init__(
            host,
            creds,
            methods,
            tls_verify,
            timeout_secs,
            request_retries,
            wait_for_ready,
            channel_options,
        )
        self._client = svc_pb2_grpc.CerbosServiceStub(self._channel)

    @handle_errors
    def check_resources(
        self,
        principal: engine_pb2.Principal,
        resources: List[request_pb2.CheckResourcesRequest.ResourceEntry],
        request_id: Union[str, None] = None,
        aux_data: Union[request_pb2.AuxData, None] = None,
    ) -> response_pb2.CheckResourcesResponse:
        """Check permissions for a list of resources

        Args:
            principal (engine_pb2.Principal): principal who is performing the action
            resources (List[request_pb2.CheckResourcesRequest.ResourceEntry]): list of resources to check permissions for
            request_id (None|str): request ID for the request (default None)
            aux_data (None|request_pb2.AuxData): auxiliary data for the request
        """
        req_id = _get_request_id(request_id)
        req = request_pb2.CheckResourcesRequest(
            request_id=req_id,
            principal=principal,
            resources=resources,
            aux_data=aux_data,
        )
        return self._client.CheckResources(req)

    def is_allowed(
        self,
        action: str,
        principal: engine_pb2.Principal,
        resource: engine_pb2.Resource,
        request_id: Union[str, None] = None,
        aux_data: Union[request_pb2.AuxData, None] = None,
    ) -> bool:
        """Check permission for a single action

        Args:
            action (str): action being performed
            principal (engine_pb2.Principal): principal who is performing the action
            resource (engine_pb2.Resource): resource on which the action is being performed
            request_id (None|str): request ID for the request (default None)
            aux_data (None|request_pb2.AuxData): auxiliary data for the request
        """
        resp = self.check_resources(
            principal=principal,
            resources=[
                request_pb2.CheckResourcesRequest.ResourceEntry(
                    actions=[action], resource=resource
                )
            ],
            request_id=request_id,
            aux_data=aux_data,
        )
        if (res := get_resource(resp, resource.id)) is not None:
            return is_allowed(res, action)
        return False

    @handle_errors
    def plan_resources(
        self,
        action: str,
        principal: engine_pb2.Principal,
        resource: engine_pb2.PlanResourcesInput.Resource,
        request_id: Union[str, None] = None,
        aux_data: Union[request_pb2.AuxData, None] = None,
    ) -> response_pb2.PlanResourcesResponse:
        """Create a query plan for performing the given action on resources of the given kind

        Args:
            action (str): Action to perform
            principal (engine_pb2.Principal): principal who is performing the action
            resource (engine_pb2.PlanResourcesInput.Resource): information about the resource kind
            request_id (None|str): request ID for the request (default None)
            aux_data (None|request_pb2.AuxData): auxiliary data for the request
        """
        req_id = _get_request_id(request_id)
        req = request_pb2.PlanResourcesRequest(
            request_id=req_id,
            action=action,
            principal=principal,
            resource=resource,
            aux_data=aux_data,
        )
        return self._client.PlanResources(req)

    def server_info(self) -> response_pb2.ServerInfoResponse:
        """Retrieve server info for the running PDP"""
        return self._client.ServerInfo(request_pb2.ServerInfoRequest())

    def with_principal(
        self,
        principal: engine_pb2.Principal,
        aux_data: Union[request_pb2.AuxData, None] = None,
    ) -> "PrincipalContext":
        """Fixes the principal for subsequent requests"""
        return PrincipalContext(client=self, principal=principal, aux_data=aux_data)


class PrincipalContext:
    """A special Cerbos client where the principal and auxData are fixed"""

    _client: CerbosClient
    _principal: engine_pb2.Principal
    _aux_data: Union[request_pb2.AuxData, None]

    def __init__(
        self,
        client: CerbosClient,
        principal: engine_pb2.Principal,
        aux_data: Union[request_pb2.AuxData, None] = None,
    ):
        self._client = client
        self._principal = principal
        self._aux_data = aux_data

    def check_resources(
        self,
        resources: List[request_pb2.CheckResourcesRequest.ResourceEntry],
        request_id: Union[str, None] = None,
    ) -> response_pb2.CheckResourcesResponse:
        """Check permissions for a list of resources

        Args:
            resources (List[request_pb2.CheckResourcesRequest.ResourceEntry]): list of resources to check permissions for
            request_id (None|str): request ID for the request (default None)
        """
        return self._client.check_resources(
            principal=self._principal,
            resources=resources,
            request_id=request_id,
            aux_data=self._aux_data,
        )

    def plan_resources(
        self,
        action: str,
        resource: engine_pb2.PlanResourcesInput.Resource,
        request_id: Union[str, None] = None,
        aux_data: Union[request_pb2.AuxData, None] = None,
    ) -> response_pb2.PlanResourcesResponse:
        """Create a query plan for performing the given action on resources of the given kind

        Args:
            action (str): Action to perform
            resource (engine_pb2.PlanResourcesInput.Resource): information about the resource kind
            request_id (None|str): request ID for the request (default None)
            aux_data (None|request_pb2.AuxData): auxiliary data for the request
        """
        return self._client.plan_resources(
            action=action,
            principal=self._principal,
            resource=resource,
            request_id=request_id,
            aux_data=aux_data,
        )

    def is_allowed(
        self,
        action: str,
        resource: engine_pb2.Resource,
        request_id: Union[str, None] = None,
    ) -> bool:
        """Check permission for a single action

        Args:
            action (str): action being performed
            resource (engine_pb2.Resource): resource on which the action is being performed
            request_id (None|str): request ID for the request (default None)
        """
        return self._client.is_allowed(
            action=action,
            principal=self._principal,
            resource=resource,
            request_id=request_id,
            aux_data=self._aux_data,
        )


def _get_request_id(request_id: Union[str, None]) -> str:
    if request_id is None:
        return str(uuid.uuid4())
    return request_id


class CerbosAdminClient(ClientBase):
    """Client for accessing the Cerbos Admin API

    Args:
        host (str): The full address of the Cerbos API server (PDP)
        admin_credentials (AdminCredentials): Mandatory basic auth credentials for accessing the Admin API. Defaults to username=`cerbos` and password=`cerbosAdmin`
        tls_verify (bool|str): If a path is passed it is used as the CA certificate. If True, we look for the path specified in `SSL_CERT_FILE` or at the default OS location. If False, server certificate verification is disabled
        timeout_secs (float): Optional request timeout in seconds (no timeout by default)
        request_retries (int): Optional maximum number of retries, including the original attempt. Anything below 2 will be treated as 0 (disabled)
        wait_for_ready (bool): Boolean specifying whether RPCs should wait until the connection is ready. Defaults to False
        channel_options (Dict[str, Any]): Optional gRPC channel options to pass on channel creation. The values need to match the expected types: https://github.com/grpc/grpc/blob/7536d8a849c0096e4c968e7730306872bb5ec674/include/grpc/impl/grpc_types.h

    Example:
        with AsyncCerbosAdminClient("localhost:3593", admin_credentials=AdminCredentials("admin", "some_password")) as cerbos:
            if cerbos.add_or_update_policy(
                "view",
                Principal(id="john", roles={"employee"}, attr={"geography": struct_pb2.Value(string_value="GB")}),
                Resource(id="XX125", "kind"="leave_request", attr={"geography": struct_pb2.Value(string_value="GB")})
                ):
                do_thing()
    """

    _client: svc_pb2_grpc.CerbosAdminServiceStub
    _creds_metadata: Tuple[Tuple[str, str]]

    def __init__(
        self,
        host: str,
        admin_credentials: Union[AdminCredentials, None] = None,
        tls_verify: TLSVerify = False,
        timeout_secs: Union[float, None] = None,
        request_retries: int = 0,
        wait_for_ready: bool = False,
        channel_options: Union[Dict[str, Any], None] = None,
    ):
        admin_credentials = admin_credentials or AdminCredentials()
        self._creds_metadata = admin_credentials.metadata()
        creds: Optional[grpc.ChannelCredentials] = None
        if tls_verify:
            cert = get_cert(tls_verify)
            creds = grpc.ssl_channel_credentials(cert)
        # {"service": "svc.CerbosAdminService", "method": "AuditLogs"},
        methods = [
            {"service": "svc.CerbosAdminService", "method": "AddOrUpdatePolicy"},
            {"service": "svc.CerbosAdminService", "method": "ListPolicies"},
            {"service": "svc.CerbosAdminService", "method": "GetPolicy"},
            {"service": "svc.CerbosAdminService", "method": "DisablePolicy"},
            {"service": "svc.CerbosAdminService", "method": "EnablePolicy"},
            {"service": "svc.CerbosAdminService", "method": "AddOrUpdateSchema"},
            {"service": "svc.CerbosAdminService", "method": "DeleteSchema"},
            {"service": "svc.CerbosAdminService", "method": "ListSchemas"},
            {"service": "svc.CerbosAdminService", "method": "GetSchema"},
            {"service": "svc.CerbosAdminService", "method": "ReloadStore"},
        ]
        super().__init__(
            host,
            creds,
            methods,
            tls_verify,
            timeout_secs,
            request_retries,
            wait_for_ready,
            channel_options,
        )
        self._client = svc_pb2_grpc.CerbosAdminServiceStub(self._channel)

    def _call(self, method, *args, **kwargs):
        """
        Wrapper to call a method with the credentials metadata attached.

        Args:
            method: The gRPC method to call.
            *args: Positional arguments to pass to the method.
            **kwargs: Keyword arguments to pass to the method.
        """
        return method(*args, metadata=self._creds_metadata, **kwargs)

    @handle_errors
    def add_or_update_policy(
        self, policies: List[policy_pb2.Policy]
    ) -> response_pb2.AddOrUpdatePolicyResponse:
        """Add or update a set of policies in the mutable store

        Args:
            policies (List[policy_pb2.Policy]): list of policies to update
        """
        req = request_pb2.AddOrUpdatePolicyRequest(policies=policies)
        return self._call(self._client.AddOrUpdatePolicy, req)

    @handle_errors
    def list_policies(
        self,
        include_disabled: bool = False,
        name_regexp: str = "",
        scope_regexp: str = "",
        version_regexp: str = "",
    ) -> response_pb2.ListPoliciesResponse:
        """List policies with optional filters

        Args:
            include_disabled (bool): include disabled policies in the response. Defaults to False
            name_regexp (str): filter the policy name with case insensitive regular expression
            scope_regexp (str): filter the policy scope with case insensitive regular expression
            version_regexp (str): filter the policy version with case insensitive regular expression
        """
        req = request_pb2.ListPoliciesRequest(
            include_disabled=include_disabled,
            name_regexp=name_regexp,
            scope_regexp=scope_regexp,
            version_regexp=version_regexp,
        )
        return self._call(self._client.ListPolicies, req)

    @handle_errors
    def get_policy(self, ids: List[str]) -> response_pb2.GetPolicyResponse:
        """Retrieve policy details for each given id

        Args:
            ids (List[str]): list of policy IDs to retrieve details for
        """
        req = request_pb2.GetPolicyRequest(id=ids)
        return self._call(self._client.GetPolicy, req)

    @handle_errors
    def disable_policy(self, ids: List[str]) -> response_pb2.DisablePolicyResponse:
        """Disable a set of policies by id

        Args:
            ids (List[str]): list of policies to disable
        """
        req = request_pb2.DisablePolicyRequest(id=ids)
        return self._call(self._client.DisablePolicy, req)

    @handle_errors
    def enable_policy(self, ids: List[str]) -> response_pb2.EnablePolicyResponse:
        """Enable a set of policies by id

        Args:
            ids (List[str]): list of policies to enable
        """
        req = request_pb2.EnablePolicyRequest(id=ids)
        return self._call(self._client.EnablePolicy, req)

    @handle_errors
    def add_or_update_schema(
        self, schemas: List[schema_pb2.Schema]
    ) -> response_pb2.AddOrUpdateSchemaResponse:
        """Add or update a set of schemas in the mutable store

        Args:
            schemas (List[schema_pb2.Schema]): list of schemas to update
        """
        req = request_pb2.AddOrUpdateSchemaRequest(schemas=schemas)
        return self._call(self._client.AddOrUpdateSchema, req)

    @handle_errors
    def delete_schema(self, ids: List[str]) -> response_pb2.DeleteSchemaResponse:
        """Delete a set of schemas by id

        Args:
            ids (List[str]): list of schemas to delete
        """
        req = request_pb2.DeleteSchemaRequest(id=ids)
        return self._call(self._client.DeleteSchema, req)

    @handle_errors
    def list_schemas(self) -> response_pb2.ListSchemasResponse:
        """List schemas"""
        req = request_pb2.ListSchemasRequest()
        return self._call(self._client.ListSchemas, req)

    @handle_errors
    def get_schema(self, ids: List[str]) -> response_pb2.GetSchemaResponse:
        """Retrieve schema details for each given id

        Args:
            ids (List[str]): list of schema IDs to retrieve details for
        """
        req = request_pb2.GetSchemaRequest(id=ids)
        return self._call(self._client.GetSchema, req)

    @handle_errors
    def reload_store(self, wait: bool = False) -> response_pb2.ReloadStoreResponse:
        """Reload the store

        Args:
            wait (bool): if true, the request will block until it finishes
        """
        req = request_pb2.ReloadStoreRequest(wait=wait)
        return self._call(self._client.ReloadStore, req)


# TODO(saml) basic auth is handled differently with the streaming API, figure out how and re-enable
# @handle_errors
# async def list_audit_logs(
#     self,
#     start_time: Union[datetime, None] = None,
#     end_time: Union[datetime, None] = None,
#     lookup: str = "",
#     tail: int = 0,
#     kind: request_pb2.ListAuditLogEntriesRequest.Kind = request_pb2.ListAuditLogEntriesRequest.KIND_ACCESS,
# ) -> response_pb2.CheckResourcesResponse:
#     """Check permissions for a list of resources
#     Args:
#         resources (List[request_pb2.CheckResourcesRequest.ResourceEntry]): list of resources to check permissions for
#         request_id (None|str): request ID for the request (default None)
#     """
#     req = request_pb2.ListAuditLogEntriesRequest(
#         kind=kind,
#     )
#     if tail > 0:
#         # req.tail = struct_pb2.Value(number_value=tail)
#         req.tail = tail
#     elif start_time:
#         if not end_time:
#             end_time = datetime.now()
#         req.between = request_pb2.ListAuditLogEntriesRequest.TimeRange(
#             start=timestamp_pb2.Timestamp(seconds=start_time.timestamp()),
#             end=timestamp_pb2.Timestamp(seconds=end_time),
#         )
#     elif lookup != "":
#         req.lookup = lookup
#     # UnaryStreamCall can't be used in 'await' expression
#     log_stream = self._client.ListAuditLogEntries(req)
#     while True:
#         resp = await log_stream.read()
#         if resp == grpc.aio.EOF:
#             break
#         print(resp.message)
#     return self._client.ListAuditLogEntries(req)



from buf.validate import validate_pb2 as _validate_pb2
from google.api import visibility_pb2 as _visibility_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class StringMatch(_message.Message):
    __slots__ = ["equals", "contains"]
    class InList(_message.Message):
        __slots__ = ["values"]
        VALUES_FIELD_NUMBER: _ClassVar[int]
        values: _containers.RepeatedScalarFieldContainer[str]
        def __init__(self, values: _Optional[_Iterable[str]] = ...) -> None: ...
    EQUALS_FIELD_NUMBER: _ClassVar[int]
    CONTAINS_FIELD_NUMBER: _ClassVar[int]
    IN_FIELD_NUMBER: _ClassVar[int]
    equals: str
    contains: str
    def __init__(self, equals: _Optional[str] = ..., contains: _Optional[str] = ..., **kwargs) -> None: ...

class FileFilter(_message.Message):
    __slots__ = ["path"]
    PATH_FIELD_NUMBER: _ClassVar[int]
    path: StringMatch
    def __init__(self, path: _Optional[_Union[StringMatch, _Mapping]] = ...) -> None: ...

class ListFilesRequest(_message.Message):
    __slots__ = ["store_id", "filter"]
    STORE_ID_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    store_id: str
    filter: FileFilter
    def __init__(self, store_id: _Optional[str] = ..., filter: _Optional[_Union[FileFilter, _Mapping]] = ...) -> None: ...

class ListFilesResponse(_message.Message):
    __slots__ = ["store_version", "files"]
    STORE_VERSION_FIELD_NUMBER: _ClassVar[int]
    FILES_FIELD_NUMBER: _ClassVar[int]
    store_version: int
    files: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, store_version: _Optional[int] = ..., files: _Optional[_Iterable[str]] = ...) -> None: ...

class GetFilesRequest(_message.Message):
    __slots__ = ["store_id", "files"]
    STORE_ID_FIELD_NUMBER: _ClassVar[int]
    FILES_FIELD_NUMBER: _ClassVar[int]
    store_id: str
    files: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, store_id: _Optional[str] = ..., files: _Optional[_Iterable[str]] = ...) -> None: ...

class File(_message.Message):
    __slots__ = ["path", "contents"]
    PATH_FIELD_NUMBER: _ClassVar[int]
    CONTENTS_FIELD_NUMBER: _ClassVar[int]
    path: str
    contents: bytes
    def __init__(self, path: _Optional[str] = ..., contents: _Optional[bytes] = ...) -> None: ...

class GetFilesResponse(_message.Message):
    __slots__ = ["store_version", "files"]
    STORE_VERSION_FIELD_NUMBER: _ClassVar[int]
    FILES_FIELD_NUMBER: _ClassVar[int]
    store_version: int
    files: _containers.RepeatedCompositeFieldContainer[File]
    def __init__(self, store_version: _Optional[int] = ..., files: _Optional[_Iterable[_Union[File, _Mapping]]] = ...) -> None: ...

class ChangeDetails(_message.Message):
    __slots__ = ["description", "uploader", "git", "internal"]
    class Git(_message.Message):
        __slots__ = ["repo", "ref", "hash", "message", "committer", "commit_date", "author", "author_date"]
        REPO_FIELD_NUMBER: _ClassVar[int]
        REF_FIELD_NUMBER: _ClassVar[int]
        HASH_FIELD_NUMBER: _ClassVar[int]
        MESSAGE_FIELD_NUMBER: _ClassVar[int]
        COMMITTER_FIELD_NUMBER: _ClassVar[int]
        COMMIT_DATE_FIELD_NUMBER: _ClassVar[int]
        AUTHOR_FIELD_NUMBER: _ClassVar[int]
        AUTHOR_DATE_FIELD_NUMBER: _ClassVar[int]
        repo: str
        ref: str
        hash: str
        message: str
        committer: str
        commit_date: _timestamp_pb2.Timestamp
        author: str
        author_date: _timestamp_pb2.Timestamp
        def __init__(self, repo: _Optional[str] = ..., ref: _Optional[str] = ..., hash: _Optional[str] = ..., message: _Optional[str] = ..., committer: _Optional[str] = ..., commit_date: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., author: _Optional[str] = ..., author_date: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...
    class Internal(_message.Message):
        __slots__ = ["source", "metadata"]
        class MetadataEntry(_message.Message):
            __slots__ = ["key", "value"]
            KEY_FIELD_NUMBER: _ClassVar[int]
            VALUE_FIELD_NUMBER: _ClassVar[int]
            key: str
            value: _struct_pb2.Value
            def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_struct_pb2.Value, _Mapping]] = ...) -> None: ...
        SOURCE_FIELD_NUMBER: _ClassVar[int]
        METADATA_FIELD_NUMBER: _ClassVar[int]
        source: str
        metadata: _containers.MessageMap[str, _struct_pb2.Value]
        def __init__(self, source: _Optional[str] = ..., metadata: _Optional[_Mapping[str, _struct_pb2.Value]] = ...) -> None: ...
    class Uploader(_message.Message):
        __slots__ = ["name", "metadata"]
        class MetadataEntry(_message.Message):
            __slots__ = ["key", "value"]
            KEY_FIELD_NUMBER: _ClassVar[int]
            VALUE_FIELD_NUMBER: _ClassVar[int]
            key: str
            value: _struct_pb2.Value
            def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_struct_pb2.Value, _Mapping]] = ...) -> None: ...
        NAME_FIELD_NUMBER: _ClassVar[int]
        METADATA_FIELD_NUMBER: _ClassVar[int]
        name: str
        metadata: _containers.MessageMap[str, _struct_pb2.Value]
        def __init__(self, name: _Optional[str] = ..., metadata: _Optional[_Mapping[str, _struct_pb2.Value]] = ...) -> None: ...
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    UPLOADER_FIELD_NUMBER: _ClassVar[int]
    GIT_FIELD_NUMBER: _ClassVar[int]
    INTERNAL_FIELD_NUMBER: _ClassVar[int]
    description: str
    uploader: ChangeDetails.Uploader
    git: ChangeDetails.Git
    internal: ChangeDetails.Internal
    def __init__(self, description: _Optional[str] = ..., uploader: _Optional[_Union[ChangeDetails.Uploader, _Mapping]] = ..., git: _Optional[_Union[ChangeDetails.Git, _Mapping]] = ..., internal: _Optional[_Union[ChangeDetails.Internal, _Mapping]] = ...) -> None: ...

class FileOp(_message.Message):
    __slots__ = ["add_or_update", "delete"]
    ADD_OR_UPDATE_FIELD_NUMBER: _ClassVar[int]
    DELETE_FIELD_NUMBER: _ClassVar[int]
    add_or_update: File
    delete: str
    def __init__(self, add_or_update: _Optional[_Union[File, _Mapping]] = ..., delete: _Optional[str] = ...) -> None: ...

class ModifyFilesRequest(_message.Message):
    __slots__ = ["store_id", "condition", "operations", "change_details"]
    class Condition(_message.Message):
        __slots__ = ["store_version_must_equal"]
        STORE_VERSION_MUST_EQUAL_FIELD_NUMBER: _ClassVar[int]
        store_version_must_equal: int
        def __init__(self, store_version_must_equal: _Optional[int] = ...) -> None: ...
    STORE_ID_FIELD_NUMBER: _ClassVar[int]
    CONDITION_FIELD_NUMBER: _ClassVar[int]
    OPERATIONS_FIELD_NUMBER: _ClassVar[int]
    CHANGE_DETAILS_FIELD_NUMBER: _ClassVar[int]
    store_id: str
    condition: ModifyFilesRequest.Condition
    operations: _containers.RepeatedCompositeFieldContainer[FileOp]
    change_details: ChangeDetails
    def __init__(self, store_id: _Optional[str] = ..., condition: _Optional[_Union[ModifyFilesRequest.Condition, _Mapping]] = ..., operations: _Optional[_Iterable[_Union[FileOp, _Mapping]]] = ..., change_details: _Optional[_Union[ChangeDetails, _Mapping]] = ...) -> None: ...

class FileError(_message.Message):
    __slots__ = ["file", "cause", "details"]
    class Cause(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        CAUSE_UNSPECIFIED: _ClassVar[FileError.Cause]
        CAUSE_INVALID_FILE_PATH: _ClassVar[FileError.Cause]
        CAUSE_UNSUPPORTED_FILE_EXTENSION: _ClassVar[FileError.Cause]
        CAUSE_INVALID_FILE_CONTENTS: _ClassVar[FileError.Cause]
        CAUSE_DUPLICATE_FILE_PATH: _ClassVar[FileError.Cause]
        CAUSE_FILE_TOO_LARGE: _ClassVar[FileError.Cause]
    CAUSE_UNSPECIFIED: FileError.Cause
    CAUSE_INVALID_FILE_PATH: FileError.Cause
    CAUSE_UNSUPPORTED_FILE_EXTENSION: FileError.Cause
    CAUSE_INVALID_FILE_CONTENTS: FileError.Cause
    CAUSE_DUPLICATE_FILE_PATH: FileError.Cause
    CAUSE_FILE_TOO_LARGE: FileError.Cause
    FILE_FIELD_NUMBER: _ClassVar[int]
    CAUSE_FIELD_NUMBER: _ClassVar[int]
    DETAILS_FIELD_NUMBER: _ClassVar[int]
    file: str
    cause: FileError.Cause
    details: str
    def __init__(self, file: _Optional[str] = ..., cause: _Optional[_Union[FileError.Cause, str]] = ..., details: _Optional[str] = ...) -> None: ...

class ModifyFilesResponse(_message.Message):
    __slots__ = ["new_store_version"]
    NEW_STORE_VERSION_FIELD_NUMBER: _ClassVar[int]
    new_store_version: int
    def __init__(self, new_store_version: _Optional[int] = ...) -> None: ...

class ReplaceFilesRequest(_message.Message):
    __slots__ = ["store_id", "condition", "zipped_contents", "files", "change_details"]
    class Condition(_message.Message):
        __slots__ = ["store_version_must_equal"]
        STORE_VERSION_MUST_EQUAL_FIELD_NUMBER: _ClassVar[int]
        store_version_must_equal: int
        def __init__(self, store_version_must_equal: _Optional[int] = ...) -> None: ...
    class Files(_message.Message):
        __slots__ = ["files"]
        FILES_FIELD_NUMBER: _ClassVar[int]
        files: _containers.RepeatedCompositeFieldContainer[File]
        def __init__(self, files: _Optional[_Iterable[_Union[File, _Mapping]]] = ...) -> None: ...
    STORE_ID_FIELD_NUMBER: _ClassVar[int]
    CONDITION_FIELD_NUMBER: _ClassVar[int]
    ZIPPED_CONTENTS_FIELD_NUMBER: _ClassVar[int]
    FILES_FIELD_NUMBER: _ClassVar[int]
    CHANGE_DETAILS_FIELD_NUMBER: _ClassVar[int]
    store_id: str
    condition: ReplaceFilesRequest.Condition
    zipped_contents: bytes
    files: ReplaceFilesRequest.Files
    change_details: ChangeDetails
    def __init__(self, store_id: _Optional[str] = ..., condition: _Optional[_Union[ReplaceFilesRequest.Condition, _Mapping]] = ..., zipped_contents: _Optional[bytes] = ..., files: _Optional[_Union[ReplaceFilesRequest.Files, _Mapping]] = ..., change_details: _Optional[_Union[ChangeDetails, _Mapping]] = ...) -> None: ...

class ErrDetailValidationFailure(_message.Message):
    __slots__ = ["errors"]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    errors: _containers.RepeatedCompositeFieldContainer[FileError]
    def __init__(self, errors: _Optional[_Iterable[_Union[FileError, _Mapping]]] = ...) -> None: ...

class ErrDetailNoUsableFiles(_message.Message):
    __slots__ = ["ignored_files"]
    IGNORED_FILES_FIELD_NUMBER: _ClassVar[int]
    ignored_files: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, ignored_files: _Optional[_Iterable[str]] = ...) -> None: ...

class ErrDetailConditionUnsatisfied(_message.Message):
    __slots__ = ["current_store_version"]
    CURRENT_STORE_VERSION_FIELD_NUMBER: _ClassVar[int]
    current_store_version: int
    def __init__(self, current_store_version: _Optional[int] = ...) -> None: ...

class ErrDetailOperationDiscarded(_message.Message):
    __slots__ = ["current_store_version", "ignored_files"]
    CURRENT_STORE_VERSION_FIELD_NUMBER: _ClassVar[int]
    IGNORED_FILES_FIELD_NUMBER: _ClassVar[int]
    current_store_version: int
    ignored_files: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, current_store_version: _Optional[int] = ..., ignored_files: _Optional[_Iterable[str]] = ...) -> None: ...

class ErrDetailCannotModifyGitConnectedStore(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ReplaceFilesResponse(_message.Message):
    __slots__ = ["new_store_version", "ignored_files"]
    NEW_STORE_VERSION_FIELD_NUMBER: _ClassVar[int]
    IGNORED_FILES_FIELD_NUMBER: _ClassVar[int]
    new_store_version: int
    ignored_files: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, new_store_version: _Optional[int] = ..., ignored_files: _Optional[_Iterable[str]] = ...) -> None: ...



# Copyright 2021-2025 Zenauth Ltd.
# SPDX-License-Identifier: Apache-2.0

import json
import os
from typing import Any, Dict, Optional

import grpc

from cerbos.sdk._async._hub._auth import _AsyncAuthClient
from cerbos.sdk._async._hub._interceptors import _AsyncAuthInterceptor
from cerbos.sdk.hub.model import Credentials

_METHOD_CONFIG: Dict[str, Any] = {
    "methodConfig": [
        {
            "name": [{}],
            "timeout": "5s",
            "maxRequestMessageBytes": 1048576,
            "retryPolicy": {
                "maxAttempts": 3,
                "initialBackoff": "0.2s",
                "maxBackoff": "5s",
                "backoffMultiplier": 2,
                "retryableStatusCodes": ["UNAVAILABLE"],
            },
        },
        {
            "name": [
                {
                    "service": "cerbos.cloud.store.v1.CerbosStoreService",
                    "method": "ReplaceFiles",
                },
                {
                    "service": "cerbos.cloud.store.v1.CerbosStoreService",
                    "method": "ModifyFiles",
                },
            ],
            "timeout": "10s",
            "maxRequestMessageBytes": 20971520,
        },
    ],
    "retryThrottling": {"maxTokens": 10, "tokenRatio": 0.5},
}


class _AsyncCerbosHubClientBase:
    _channel: grpc.aio.Channel
    _timeout_secs: float = 5

    def __init__(
        self,
        credentials: Optional[Credentials] = None,
        api_endpoint: Optional[str] = None,
        timeout_secs: Optional[float] = None,
    ):
        target = os.getenv("CERBOS_HUB_API_ENDPOINT", "dns:///api.cerbos.cloud")
        if api_endpoint:
            target = api_endpoint

        options = [
            ("grpc.enable_retries", 1),
            ("grpc.service_config_disable_resolution", 0),
            ("grpc.service_config", json.dumps(_METHOD_CONFIG)),
        ]

        if timeout_secs:
            self._timeout_secs = timeout_secs

        channel = grpc.aio.secure_channel(
            target,
            credentials=grpc.ssl_channel_credentials(),
            options=options,
        )
        auth_interceptor = _AsyncAuthInterceptor(
            _AsyncAuthClient(channel, self._timeout_secs, credentials)
        )

        # We have to create a new channel here because aio.channels have no way to add interceptors after creation.
        self._channel = grpc.aio.secure_channel(
            target,
            credentials=grpc.ssl_channel_credentials(),
            options=options,
            interceptors=(auth_interceptor,),
        )

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        await self.close()

    async def close(self):
        await self._channel.close()


# Because grpc.aio is implemented differently, unasync can't do a direct translation. We need a second implementation here.


class _CerbosHubClientBase:
    _channel: grpc.Channel
    _timeout_secs: float = 5

    def __init__(
        self,
        credentials: Optional[Credentials] = None,
        api_endpoint: Optional[str] = None,
        timeout_secs: Optional[float] = None,
    ):
        target = os.getenv("CERBOS_HUB_API_ENDPOINT", "dns:///api.cerbos.cloud")
        if api_endpoint:
            target = api_endpoint

        options = [
            ("grpc.enable_retries", 1),
            ("grpc.service_config_disable_resolution", 0),
            ("grpc.service_config", json.dumps(_METHOD_CONFIG)),
        ]

        if timeout_secs:
            self._timeout_secs = timeout_secs

        channel = grpc.secure_channel(
            target,
            credentials=grpc.ssl_channel_credentials(),
            options=options,
        )
        auth_interceptor = _AuthInterceptor(  # noqa: F821
            _AuthClient(channel, self._timeout_secs, credentials)  # noqa: F821
        )
        self._channel = grpc.intercept_channel(channel, auth_interceptor)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def close(self):
        self._channel.close()



# Copyright 2021-2025 Zenauth Ltd.
# SPDX-License-Identifier: Apache-2.0

from functools import wraps
from typing import Iterable, List, Optional, Union

import circuitbreaker
import grpc
from google.protobuf import json_format
from google.rpc import code_pb2
from grpc_status import rpc_status

from cerbos.cloud.store.v1 import store_pb2, store_pb2_grpc
from cerbos.sdk._async._hub._auth import InvalidCredentialsError
from cerbos.sdk._async._hub._client import _AsyncCerbosHubClientBase
from cerbos.sdk.hub.model import Credentials
from cerbos.sdk.hub.store_model import (
    AbortedError,
    AuthenticationFailedError,
    CannotModifyGitConnectedStoreError,
    ChangeDetails,
    ConditionUnsatisfiedError,
    File,
    FileOps,
    FilterPathContains,
    FilterPathEqual,
    FilterPathIn,
    GetFilesResponse,
    InvalidRequestError,
    ListFilesResponse,
    ModifyFilesResponse,
    NoUsableFilesError,
    OperationDiscardedError,
    PermissionDeniedError,
    ReplaceFilesResponse,
    StoreNotFoundError,
    TooManyFailuresError,
    UnknownError,
    ValidationFailureError,
)

_MAX_FILE_SIZE = 5 * 1024 * 1024
_MAX_UPLOAD_SIZE = 50 * 1024 * 1024
_MAX_ZIP_SIZE = 15 * 1024 * 1024
_MIN_ZIP_SIZE = 22


def _exclude_errors(thrown_type, thrown_value):
    if isinstance(thrown_value, grpc.RpcError):
        status = rpc_status.from_call(thrown_value)
        if status is None:
            return False

        return status.code in {
            code_pb2.ABORTED,
            code_pb2.CANCELLED,
            code_pb2.DEADLINE_EXCEEDED,
        }

    return False


class CircuitBreaker(circuitbreaker.CircuitBreaker):
    FAILURE_THRESHOLD = 10
    RECOVERY_TIMEOUT = 60
    EXPECTED_EXCEPTION = _exclude_errors


_CIRCUIT_BREAKER = CircuitBreaker().decorate


def handle_store_errors(method):
    @wraps(method)
    async def wrapper(*args, **kwargs):
        try:
            return await method(*args, **kwargs)
        except InvalidCredentialsError as e:
            raise AuthenticationFailedError(e)
        except circuitbreaker.CircuitBreakerError as e:
            raise TooManyFailuresError(e)
        except grpc.RpcError as e:
            status = rpc_status.from_call(e)
            if status is None:
                raise UnknownError(e)

            if status.code == code_pb2.ABORTED:
                raise AbortedError(e)
            elif status.code == code_pb2.ALREADY_EXISTS:
                for detail in status.details:
                    if detail.Is(store_pb2.ErrDetailOperationDiscarded.DESCRIPTOR):
                        info = store_pb2.ErrDetailOperationDiscarded()
                        detail.Unpack(info)
                        raise OperationDiscardedError(e, info)

                raise OperationDiscardedError(e)

            elif status.code == code_pb2.CANCELLED:
                raise AbortedError(e)
            elif status.code == code_pb2.DEADLINE_EXCEEDED:
                raise AbortedError(e)
            elif status.code == code_pb2.FAILED_PRECONDITION:
                for detail in status.details:
                    if detail.Is(
                        store_pb2.ErrDetailCannotModifyGitConnectedStore.DESCRIPTOR
                    ):
                        raise CannotModifyGitConnectedStoreError(e)

                    if detail.Is(store_pb2.ErrDetailConditionUnsatisfied.DESCRIPTOR):
                        info = store_pb2.ErrDetailConditionUnsatisfied()
                        detail.Unpack(info)
                        raise ConditionUnsatisfiedError(e, info)
            elif status.code == code_pb2.INVALID_ARGUMENT:
                for detail in status.details:
                    if detail.Is(store_pb2.ErrDetailNoUsableFiles.DESCRIPTOR):
                        info = store_pb2.ErrDetailNoUsableFiles()
                        detail.Unpack(info)
                        raise NoUsableFilesError(e, info)

                    if detail.Is(store_pb2.ErrDetailValidationFailure.DESCRIPTOR):
                        info = store_pb2.ErrDetailValidationFailure()
                        detail.Unpack(info)
                        raise ValidationFailureError(e, info)
            elif status.code == code_pb2.NOT_FOUND:
                raise StoreNotFoundError(e)
            elif status.code == code_pb2.PERMISSION_DENIED:
                raise PermissionDeniedError(e)
            else:
                raise UnknownError(e)

    return wrapper


class AsyncCerbosHubStoreClient(_AsyncCerbosHubClientBase):
    """
    Client for working with Cerbos Hub stores.
    Requires a set of credentials which can be generated using the Cerbos Hub interface.
    Provide the credentials using the environment variables CERBOS_HUB_CLIENT_ID and CERBOS_HUB_CLIENT_SECRET.

    For well-known errors, the methods of this class raise errors that are subclasses of cerbos.sdk.hub.store_model.RpcError.
    Some of the error subclasses contain additional details about the error such as validation failures and ignored files.
    - AbortedError: call cancelled or timed out
    - AuthenticationFailedError: invalid credentials
    - CannotModifyGitConnectedStoreError: attempting to modify a store connected to Git
    - ConditionUnsatisfiedError: the condition attached to the call wasn't satisfied
    - InvalidRequestError: the arguments passed to the call are invalid
    - NoUsableFilesError: none of the provided files can be stored
    - OperationDiscardedError: the operation does not modify store state
    - PermissionDeniedError: permission denied for the action
    - TooManyFailuresError: the client has had too many failed requests and should cool down
    - UnknownError: catch-all for errors that are not specifically handled
    - ValidationFailureError: the request was rejected due to validation failures
    """

    _store_stub: store_pb2_grpc.CerbosStoreServiceStub

    def __init__(
        self,
        credentials: Optional[Credentials] = None,
        api_endpoint: Optional[str] = None,
        timeout_secs: Optional[float] = None,
    ):
        super(AsyncCerbosHubStoreClient, self).__init__(
            credentials, api_endpoint, timeout_secs
        )
        self._store_stub = store_pb2_grpc.CerbosStoreServiceStub(self._channel)

    @handle_store_errors
    @_CIRCUIT_BREAKER
    async def replace_files(
        self,
        store_id: str,
        message: str,
        contents: Union[bytes, Iterable[File]],
        version_must_equal: Optional[int] = None,
        change_details: Optional[ChangeDetails] = None,
    ) -> ReplaceFilesResponse:
        """
        Overwrite the store such that it only contains the files included in this request.
        Raises OperationDiscardedError if the store is already at the desired state.
        """

        if not (store_id and store_id.strip()):
            raise InvalidRequestError(ValueError("store_id is required"))

        _change_details: Optional[store_pb2.ChangeDetails] = None
        if change_details is None:
            _change_details = store_pb2.ChangeDetails(
                description=message,
                uploader=store_pb2.ChangeDetails.Uploader(name="cerbos-sdk-python"),
            )
        else:
            _change_details = change_details.raw

        _condition: Optional[store_pb2.ReplaceFilesRequest.Condition] = None
        if version_must_equal:
            _condition = store_pb2.ReplaceFilesRequest.Condition(
                store_version_must_equal=version_must_equal
            )

        req = store_pb2.ReplaceFilesRequest(
            store_id=store_id, condition=_condition, change_details=_change_details
        )
        req: store_pb2.ReplaceFilesRequest
        if isinstance(contents, bytes):
            if not (_MIN_ZIP_SIZE <= len(contents) <= _MAX_ZIP_SIZE):
                raise InvalidRequestError(ValueError("invalid zip contents"))

            req = store_pb2.ReplaceFilesRequest(
                store_id=store_id,
                condition=_condition,
                change_details=_change_details,
                zipped_contents=contents,
            )
        else:
            files = []
            total_size = 0
            for f in contents:
                file_size = len(f.contents)
                if file_size > _MAX_FILE_SIZE:
                    raise InvalidRequestError(ValueError(f"file {f.path} is too large"))

                total_size += file_size
                if total_size > _MAX_UPLOAD_SIZE:
                    raise InvalidRequestError(
                        ValueError("total size of files exceeds upload limit")
                    )

                files.append(store_pb2.File(path=f.path, contents=f.contents))

            if len(files) == 0:
                raise InvalidRequestError(ValueError("file list is empty"))

            req = store_pb2.ReplaceFilesRequest(
                store_id=store_id,
                condition=_condition,
                change_details=_change_details,
                files=store_pb2.ReplaceFilesRequest.Files(files=files),
            )

        resp = await self._store_stub.ReplaceFiles(req)
        return ReplaceFilesResponse(resp)

    @handle_store_errors
    @_CIRCUIT_BREAKER
    async def replace_files_lenient(
        self,
        store_id: str,
        message: str,
        contents: Union[bytes, Iterable[File]],
        version_must_equal: Optional[int] = None,
        change_details: Optional[ChangeDetails] = None,
    ) -> ReplaceFilesResponse:
        """
        Overwrite the store such that it only contains the files included in this request.
        Does not raise OperationDiscardedError if the store is already at the desired state.
        """
        try:
            return await self.replace_files(
                store_id, message, contents, version_must_equal, change_details
            )
        except OperationDiscardedError as e:
            return ReplaceFilesResponse(
                store_pb2.ReplaceFilesResponse(
                    new_store_version=e.current_store_version
                )
            )

    @handle_store_errors
    @_CIRCUIT_BREAKER
    async def modify_files(
        self,
        store_id: str,
        message: str,
        file_ops: FileOps,
        version_must_equal: Optional[int] = None,
        change_details: Optional[ChangeDetails] = None,
    ) -> ModifyFilesResponse:
        """
        Add or delete files.
        Raises OperationDiscardedError if the operation does not change store state.
        """

        if not (store_id and store_id.strip()):
            raise InvalidRequestError(ValueError("store_id is required"))

        _change_details: Optional[store_pb2.ChangeDetails] = None
        if change_details is None:
            _change_details = store_pb2.ChangeDetails(
                description=message,
                uploader=store_pb2.ChangeDetails.Uploader(name="cerbos-sdk-python"),
            )
        else:
            _change_details = change_details.raw

        _condition: Optional[store_pb2.ModifyFilesRequest.Condition] = None
        if version_must_equal:
            _condition = store_pb2.ModifyFilesRequest.Condition(
                store_version_must_equal=version_must_equal
            )

        ops: List[store_pb2.FileOp] = []
        if file_ops.add:
            total_size = 0
            for f in file_ops.add:
                file_size = len(f.contents)
                if file_size > _MAX_FILE_SIZE:
                    raise InvalidRequestError(ValueError(f"file {f.path} is too large"))

                total_size += file_size
                if total_size > _MAX_UPLOAD_SIZE:
                    raise InvalidRequestError(
                        ValueError("total size of files exceeds upload limit")
                    )

                ops.append(
                    store_pb2.FileOp(
                        add_or_update=store_pb2.File(path=f.path, contents=f.contents)
                    )
                )
        if file_ops.delete:
            for f in file_ops.delete:
                ops.append(store_pb2.FileOp(delete=f))

        if len(ops) == 0:
            raise InvalidRequestError(ValueError("file operations list is empty"))

        req = store_pb2.ModifyFilesRequest(
            store_id=store_id,
            condition=_condition,
            operations=ops,
            change_details=_change_details,
        )
        resp = await self._store_stub.ModifyFiles(req)
        return ModifyFilesResponse(resp)

    @handle_store_errors
    @_CIRCUIT_BREAKER
    async def modify_files_lenient(
        self,
        store_id: str,
        message: str,
        file_ops: FileOps,
        version_must_equal: Optional[int] = None,
        change_details: Optional[ChangeDetails] = None,
    ) -> ModifyFilesResponse:
        """
        Add or delete files.
        Does not raise OperationDiscardedError if the operation does not change store state.
        """
        try:
            return await self.modify_files(
                store_id, message, file_ops, version_must_equal, change_details
            )
        except OperationDiscardedError as e:
            return ModifyFilesResponse(
                store_pb2.ModifyFilesResponse(new_store_version=e.current_store_version)
            )

    @handle_store_errors
    @_CIRCUIT_BREAKER
    async def get_files(
        self, store_id: str, file_paths: Iterable[str]
    ) -> GetFilesResponse:
        """
        Retrieve the contents of the given files.
        """
        if not (store_id and store_id.strip()):
            raise InvalidRequestError(ValueError("store_id is required"))

        req = store_pb2.GetFilesRequest(store_id=store_id, files=file_paths)
        resp = await self._store_stub.GetFiles(req)
        return GetFilesResponse(resp)

    @handle_store_errors
    @circuitbreaker.circuit(cls=CircuitBreaker)
    async def list_files(
        self,
        store_id: str,
        filter: Optional[
            Union[FilterPathEqual, FilterPathContains, FilterPathIn]
        ] = None,
    ) -> ListFilesResponse:
        """
        List the files available on the remote store.
        The listing can be filtered by providing an optional `filter` argument which can be one of the following types.
          - FilterPathEqual: Match a path exactly
          - FilterPathLike: Match a path partially
          - FilterPathIn: Match any or all of the paths in the given list
        """
        path_filter: Optional[store_pb2.FileFilter] = None
        if filter:
            if isinstance(filter, FilterPathEqual):
                path_filter = store_pb2.FileFilter(
                    path=store_pb2.StringMatch(equals=filter.path)
                )
            elif isinstance(filter, FilterPathContains):
                path_filter = store_pb2.FileFilter(
                    path=store_pb2.StringMatch(contains=filter.fragment)
                )
            elif isinstance(filter, FilterPathIn):
                # "in" is a Python keyword  :(
                path_filter = json_format.ParseDict(
                    {"path": {"in": {"values": [p for p in filter.paths]}}},
                    store_pb2.FileFilter(),
                )

        resp = await self._store_stub.ListFiles(
            store_pb2.ListFilesRequest(store_id=store_id, filter=path_filter)
        )
        return ListFilesResponse(resp)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from cerbos.cloud.store.v1 import store_pb2 as cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2


class CerbosStoreServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListFiles = channel.unary_unary(
                '/cerbos.cloud.store.v1.CerbosStoreService/ListFiles',
                request_serializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ListFilesRequest.SerializeToString,
                response_deserializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ListFilesResponse.FromString,
                _registered_method=True)
        self.GetFiles = channel.unary_unary(
                '/cerbos.cloud.store.v1.CerbosStoreService/GetFiles',
                request_serializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.GetFilesRequest.SerializeToString,
                response_deserializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.GetFilesResponse.FromString,
                _registered_method=True)
        self.ModifyFiles = channel.unary_unary(
                '/cerbos.cloud.store.v1.CerbosStoreService/ModifyFiles',
                request_serializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ModifyFilesRequest.SerializeToString,
                response_deserializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ModifyFilesResponse.FromString,
                _registered_method=True)
        self.ReplaceFiles = channel.unary_unary(
                '/cerbos.cloud.store.v1.CerbosStoreService/ReplaceFiles',
                request_serializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ReplaceFilesRequest.SerializeToString,
                response_deserializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ReplaceFilesResponse.FromString,
                _registered_method=True)


class CerbosStoreServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ListFiles(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFiles(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ModifyFiles(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReplaceFiles(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_CerbosStoreServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ListFiles': grpc.unary_unary_rpc_method_handler(
                    servicer.ListFiles,
                    request_deserializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ListFilesRequest.FromString,
                    response_serializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ListFilesResponse.SerializeToString,
            ),
            'GetFiles': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFiles,
                    request_deserializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.GetFilesRequest.FromString,
                    response_serializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.GetFilesResponse.SerializeToString,
            ),
            'ModifyFiles': grpc.unary_unary_rpc_method_handler(
                    servicer.ModifyFiles,
                    request_deserializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ModifyFilesRequest.FromString,
                    response_serializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ModifyFilesResponse.SerializeToString,
            ),
            'ReplaceFiles': grpc.unary_unary_rpc_method_handler(
                    servicer.ReplaceFiles,
                    request_deserializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ReplaceFilesRequest.FromString,
                    response_serializer=cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ReplaceFilesResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cerbos.cloud.store.v1.CerbosStoreService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('cerbos.cloud.store.v1.CerbosStoreService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class CerbosStoreService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ListFiles(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/cerbos.cloud.store.v1.CerbosStoreService/ListFiles',
            cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ListFilesRequest.SerializeToString,
            cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ListFilesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetFiles(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/cerbos.cloud.store.v1.CerbosStoreService/GetFiles',
            cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.GetFilesRequest.SerializeToString,
            cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.GetFilesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ModifyFiles(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/cerbos.cloud.store.v1.CerbosStoreService/ModifyFiles',
            cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ModifyFilesRequest.SerializeToString,
            cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ModifyFilesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ReplaceFiles(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/cerbos.cloud.store.v1.CerbosStoreService/ReplaceFiles',
            cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ReplaceFilesRequest.SerializeToString,
            cerbos_dot_cloud_dot_store_dot_v1_dot_store__pb2.ReplaceFilesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



from buf.validate import validate_pb2 as _validate_pb2
from cerbos.cloud.pdp.v1 import pdp_pb2 as _pdp_pb2
from google.api import visibility_pb2 as _visibility_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Source(_message.Message):
    __slots__ = ["deployment_id", "playground_id"]
    DEPLOYMENT_ID_FIELD_NUMBER: _ClassVar[int]
    PLAYGROUND_ID_FIELD_NUMBER: _ClassVar[int]
    deployment_id: str
    playground_id: str
    def __init__(self, deployment_id: _Optional[str] = ..., playground_id: _Optional[str] = ...) -> None: ...

class BundleInfo(_message.Message):
    __slots__ = ["source", "input_hash", "output_hash", "encryption_key", "segments"]
    class Segment(_message.Message):
        __slots__ = ["segment_id", "checksum", "download_urls"]
        SEGMENT_ID_FIELD_NUMBER: _ClassVar[int]
        CHECKSUM_FIELD_NUMBER: _ClassVar[int]
        DOWNLOAD_URLS_FIELD_NUMBER: _ClassVar[int]
        segment_id: int
        checksum: bytes
        download_urls: _containers.RepeatedScalarFieldContainer[str]
        def __init__(self, segment_id: _Optional[int] = ..., checksum: _Optional[bytes] = ..., download_urls: _Optional[_Iterable[str]] = ...) -> None: ...
    SOURCE_FIELD_NUMBER: _ClassVar[int]
    INPUT_HASH_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_HASH_FIELD_NUMBER: _ClassVar[int]
    ENCRYPTION_KEY_FIELD_NUMBER: _ClassVar[int]
    SEGMENTS_FIELD_NUMBER: _ClassVar[int]
    source: Source
    input_hash: bytes
    output_hash: bytes
    encryption_key: bytes
    segments: _containers.RepeatedCompositeFieldContainer[BundleInfo.Segment]
    def __init__(self, source: _Optional[_Union[Source, _Mapping]] = ..., input_hash: _Optional[bytes] = ..., output_hash: _Optional[bytes] = ..., encryption_key: _Optional[bytes] = ..., segments: _Optional[_Iterable[_Union[BundleInfo.Segment, _Mapping]]] = ...) -> None: ...

class Meta(_message.Message):
    __slots__ = ["bundle_id", "source"]
    BUNDLE_ID_FIELD_NUMBER: _ClassVar[int]
    SOURCE_FIELD_NUMBER: _ClassVar[int]
    bundle_id: str
    source: str
    def __init__(self, bundle_id: _Optional[str] = ..., source: _Optional[str] = ...) -> None: ...

class Manifest(_message.Message):
    __slots__ = ["api_version", "policy_index", "schemas", "meta"]
    class PolicyIndexEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    API_VERSION_FIELD_NUMBER: _ClassVar[int]
    POLICY_INDEX_FIELD_NUMBER: _ClassVar[int]
    SCHEMAS_FIELD_NUMBER: _ClassVar[int]
    META_FIELD_NUMBER: _ClassVar[int]
    api_version: str
    policy_index: _containers.ScalarMap[str, str]
    schemas: _containers.RepeatedScalarFieldContainer[str]
    meta: Meta
    def __init__(self, api_version: _Optional[str] = ..., policy_index: _Optional[_Mapping[str, str]] = ..., schemas: _Optional[_Iterable[str]] = ..., meta: _Optional[_Union[Meta, _Mapping]] = ...) -> None: ...

class GetBundleRequest(_message.Message):
    __slots__ = ["pdp_id", "source"]
    PDP_ID_FIELD_NUMBER: _ClassVar[int]
    SOURCE_FIELD_NUMBER: _ClassVar[int]
    pdp_id: _pdp_pb2.Identifier
    source: Source
    def __init__(self, pdp_id: _Optional[_Union[_pdp_pb2.Identifier, _Mapping]] = ..., source: _Optional[_Union[Source, _Mapping]] = ...) -> None: ...

class GetBundleResponse(_message.Message):
    __slots__ = ["bundle_info"]
    BUNDLE_INFO_FIELD_NUMBER: _ClassVar[int]
    bundle_info: BundleInfo
    def __init__(self, bundle_info: _Optional[_Union[BundleInfo, _Mapping]] = ...) -> None: ...

class WatchBundleRequest(_message.Message):
    __slots__ = ["pdp_id", "start", "heartbeat"]
    class Start(_message.Message):
        __slots__ = ["source"]
        SOURCE_FIELD_NUMBER: _ClassVar[int]
        source: Source
        def __init__(self, source: _Optional[_Union[Source, _Mapping]] = ...) -> None: ...
    class Heartbeat(_message.Message):
        __slots__ = ["timestamp", "active_bundle_id"]
        TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
        ACTIVE_BUNDLE_ID_FIELD_NUMBER: _ClassVar[int]
        timestamp: _timestamp_pb2.Timestamp
        active_bundle_id: str
        def __init__(self, timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., active_bundle_id: _Optional[str] = ...) -> None: ...
    PDP_ID_FIELD_NUMBER: _ClassVar[int]
    START_FIELD_NUMBER: _ClassVar[int]
    HEARTBEAT_FIELD_NUMBER: _ClassVar[int]
    pdp_id: _pdp_pb2.Identifier
    start: WatchBundleRequest.Start
    heartbeat: WatchBundleRequest.Heartbeat
    def __init__(self, pdp_id: _Optional[_Union[_pdp_pb2.Identifier, _Mapping]] = ..., start: _Optional[_Union[WatchBundleRequest.Start, _Mapping]] = ..., heartbeat: _Optional[_Union[WatchBundleRequest.Heartbeat, _Mapping]] = ...) -> None: ...

class WatchBundleResponse(_message.Message):
    __slots__ = ["bundle_update", "reconnect", "bundle_removed"]
    class Reconnect(_message.Message):
        __slots__ = ["backoff", "reason"]
        BACKOFF_FIELD_NUMBER: _ClassVar[int]
        REASON_FIELD_NUMBER: _ClassVar[int]
        backoff: _duration_pb2.Duration
        reason: str
        def __init__(self, backoff: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., reason: _Optional[str] = ...) -> None: ...
    class BundleRemoved(_message.Message):
        __slots__ = []
        def __init__(self) -> None: ...
    BUNDLE_UPDATE_FIELD_NUMBER: _ClassVar[int]
    RECONNECT_FIELD_NUMBER: _ClassVar[int]
    BUNDLE_REMOVED_FIELD_NUMBER: _ClassVar[int]
    bundle_update: BundleInfo
    reconnect: WatchBundleResponse.Reconnect
    bundle_removed: WatchBundleResponse.BundleRemoved
    def __init__(self, bundle_update: _Optional[_Union[BundleInfo, _Mapping]] = ..., reconnect: _Optional[_Union[WatchBundleResponse.Reconnect, _Mapping]] = ..., bundle_removed: _Optional[_Union[WatchBundleResponse.BundleRemoved, _Mapping]] = ...) -> None: ...



from buf.validate import validate_pb2 as _validate_pb2
from cerbos.audit.v1 import audit_pb2 as _audit_pb2
from cerbos.cloud.pdp.v1 import pdp_pb2 as _pdp_pb2
from google.api import visibility_pb2 as _visibility_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class IngestBatch(_message.Message):
    __slots__ = ["id", "entries"]
    class EntryKind(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        ENTRY_KIND_UNSPECIFIED: _ClassVar[IngestBatch.EntryKind]
        ENTRY_KIND_ACCESS_LOG: _ClassVar[IngestBatch.EntryKind]
        ENTRY_KIND_DECISION_LOG: _ClassVar[IngestBatch.EntryKind]
    ENTRY_KIND_UNSPECIFIED: IngestBatch.EntryKind
    ENTRY_KIND_ACCESS_LOG: IngestBatch.EntryKind
    ENTRY_KIND_DECISION_LOG: IngestBatch.EntryKind
    class Entry(_message.Message):
        __slots__ = ["kind", "timestamp", "access_log_entry", "decision_log_entry"]
        KIND_FIELD_NUMBER: _ClassVar[int]
        TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
        ACCESS_LOG_ENTRY_FIELD_NUMBER: _ClassVar[int]
        DECISION_LOG_ENTRY_FIELD_NUMBER: _ClassVar[int]
        kind: IngestBatch.EntryKind
        timestamp: _timestamp_pb2.Timestamp
        access_log_entry: _audit_pb2.AccessLogEntry
        decision_log_entry: _audit_pb2.DecisionLogEntry
        def __init__(self, kind: _Optional[_Union[IngestBatch.EntryKind, str]] = ..., timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., access_log_entry: _Optional[_Union[_audit_pb2.AccessLogEntry, _Mapping]] = ..., decision_log_entry: _Optional[_Union[_audit_pb2.DecisionLogEntry, _Mapping]] = ...) -> None: ...
    ID_FIELD_NUMBER: _ClassVar[int]
    ENTRIES_FIELD_NUMBER: _ClassVar[int]
    id: str
    entries: _containers.RepeatedCompositeFieldContainer[IngestBatch.Entry]
    def __init__(self, id: _Optional[str] = ..., entries: _Optional[_Iterable[_Union[IngestBatch.Entry, _Mapping]]] = ...) -> None: ...

class IngestRequest(_message.Message):
    __slots__ = ["pdp_id", "batch"]
    PDP_ID_FIELD_NUMBER: _ClassVar[int]
    BATCH_FIELD_NUMBER: _ClassVar[int]
    pdp_id: _pdp_pb2.Identifier
    batch: IngestBatch
    def __init__(self, pdp_id: _Optional[_Union[_pdp_pb2.Identifier, _Mapping]] = ..., batch: _Optional[_Union[IngestBatch, _Mapping]] = ...) -> None: ...

class IngestResponse(_message.Message):
    __slots__ = ["success", "backoff"]
    class Backoff(_message.Message):
        __slots__ = ["duration"]
        DURATION_FIELD_NUMBER: _ClassVar[int]
        duration: _duration_pb2.Duration
        def __init__(self, duration: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ...) -> None: ...
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    BACKOFF_FIELD_NUMBER: _ClassVar[int]
    success: _empty_pb2.Empty
    backoff: IngestResponse.Backoff
    def __init__(self, success: _Optional[_Union[_empty_pb2.Empty, _Mapping]] = ..., backoff: _Optional[_Union[IngestResponse.Backoff, _Mapping]] = ...) -> None: ...



# Copyright 2021-2025 Zenauth Ltd.
# SPDX-License-Identifier: Apache-2.0

import datetime
from dataclasses import dataclass
from enum import Enum, auto
from typing import Iterable, List, Mapping, Optional, Self

from google.protobuf import json_format, struct_pb2, timestamp_pb2

from cerbos.cloud.store.v1 import store_pb2


class RpcErrorCause(Enum):
    ABORTED = auto()
    AUTHENTICATION_FAILED = auto()
    CANNOT_MODIFY_GIT_CONNECTED_STORE = auto()
    CONDITION_UNSATISFIED = auto()
    INVALID_REQUEST = auto()
    NO_USABLE_FILES = auto()
    OPERATION_DISCARDED = auto()
    PERMISSION_DENIED = auto()
    STORE_NOT_FOUND = auto()
    TOO_MANY_FAILURES = auto()
    UNKNOWN = auto()
    VALIDATION_FAILURE = auto()

    def __str__(self):
        return self.name


class RpcError(Exception):
    cause: RpcErrorCause
    underlying: Exception

    def __init__(self, cause: RpcErrorCause, underlying: Exception):
        msg = "RPC error: {0}".format(cause)
        super(RpcError, self).__init__(msg)
        self.cause = cause
        self.underlying = underlying


class AbortedError(RpcError):
    def __init__(self, underlying: Exception):
        super(AbortedError, self).__init__(RpcErrorCause.ABORTED, underlying)


class AuthenticationFailedError(RpcError):
    def __init__(self, underlying: Exception):
        super(AuthenticationFailedError, self).__init__(
            RpcErrorCause.AUTHENTICATION_FAILED, underlying
        )


class CannotModifyGitConnectedStoreError(RpcError):
    def __init__(self, underlying: Exception):
        super(CannotModifyGitConnectedStoreError, self).__init__(
            RpcErrorCause.CANNOT_MODIFY_GIT_CONNECTED_STORE, underlying
        )


class ConditionUnsatisfiedError(RpcError):
    current_store_version: Optional[int] = None

    def __init__(
        self,
        underlying: Exception,
        details: Optional[store_pb2.ErrDetailConditionUnsatisfied] = None,
    ):
        super(ConditionUnsatisfiedError, self).__init__(
            RpcErrorCause.CONDITION_UNSATISFIED, underlying
        )
        if details:
            self.current_store_version = details.current_store_version


class InvalidRequestError(RpcError):
    def __init__(self, underlying: Exception):
        super(InvalidRequestError, self).__init__(
            RpcErrorCause.INVALID_REQUEST, underlying
        )


class NoUsableFilesError(RpcError):
    ignored_files: Optional[Iterable[str]]

    def __init__(
        self, underlying: Exception, details: store_pb2.ErrDetailNoUsableFiles
    ):
        super(NoUsableFilesError, self).__init__(
            RpcErrorCause.NO_USABLE_FILES, underlying
        )

        if details.ignored_files:
            self.ignored_files = details.ignored_files


class OperationDiscardedError(RpcError):
    current_store_version: Optional[int] = None
    ignored_files: Optional[Iterable[str]] = None

    def __init__(
        self,
        underlying: Exception,
        details: Optional[store_pb2.ErrDetailOperationDiscarded] = None,
    ):
        super(OperationDiscardedError, self).__init__(
            RpcErrorCause.OPERATION_DISCARDED, underlying
        )
        if details:
            self.current_store_version = details.current_store_version
            self.ignored_files = details.ignored_files


class PermissionDeniedError(RpcError):
    def __init__(self, underlying: Exception):
        super(PermissionDeniedError, self).__init__(
            RpcErrorCause.PERMISSION_DENIED, underlying
        )


class StoreNotFoundError(RpcError):
    def __init__(self, underlying: Exception):
        super(StoreNotFoundError, self).__init__(
            RpcErrorCause.STORE_NOT_FOUND, underlying
        )


class TooManyFailuresError(RpcError):
    def __init__(self, underlying: Exception):
        super(TooManyFailuresError, self).__init__(
            RpcErrorCause.TOO_MANY_FAILURES, underlying
        )


class UnknownError(RpcError):
    def __init__(self, underlying: Exception):
        super(UnknownError, self).__init__(RpcErrorCause.UNKNOWN, underlying)


class ValidationFailureError(RpcError):
    errors: Optional[Iterable[store_pb2.FileError]]

    def __init__(
        self, underlying: Exception, details: store_pb2.ErrDetailValidationFailure
    ):
        super(ValidationFailureError, self).__init__(
            RpcErrorCause.VALIDATION_FAILURE, underlying
        )

        if details.errors:
            self.errors = details.errors


class ChangeDetails:
    """
    Provide detailed metadata about the change being commited to the store.
    """

    raw: store_pb2.ChangeDetails

    def __init__(self, description: str):
        self.raw = store_pb2.ChangeDetails(description=description)

    def with_uploader(
        self, name: str, metadata: Optional[Mapping[str, struct_pb2.Value]] = None
    ) -> Self:
        """
        Set the name and any custom metadata about the uploader.
        """
        if metadata:
            self.raw.MergeFrom(
                store_pb2.ChangeDetails(
                    uploader=store_pb2.ChangeDetails.Uploader(
                        name=name, metadata=metadata
                    )
                )
            )
        else:
            self.raw.MergeFrom(
                store_pb2.ChangeDetails(
                    uploader=store_pb2.ChangeDetails.Uploader(name=name)
                )
            )

        return self

    def with_git_source(
        self,
        repo: str,
        hash: str,
        ref: Optional[str] = None,
        message: Optional[str] = None,
        committer: Optional[str] = None,
        commit_date: Optional[datetime.datetime] = None,
        author: Optional[str] = None,
        author_date: Optional[datetime.datetime] = None,
    ) -> Self:
        """
        Attach information about the underlying git commit.
        Mutually exclusive with internal source.
        """
        _commit_date: Optional[timestamp_pb2.Timestamp] = None
        if commit_date:
            _commit_date = timestamp_pb2.Timestamp()
            _commit_date.FromDatetime(commit_date)

        _author_date: Optional[timestamp_pb2.Timestamp] = None
        if author_date:
            _author_date = timestamp_pb2.Timestamp()
            _author_date.FromDatetime(author_date)

        self.raw.MergeFrom(
            store_pb2.ChangeDetails(
                git=store_pb2.ChangeDetails.Git(
                    repo=repo,
                    ref=ref,
                    hash=hash,
                    message=message,
                    committer=committer,
                    commit_date=_commit_date,
                    author=author,
                    author_date=_author_date,
                )
            )
        )

        return self

    def with_internal_source(
        self, source: str, metadata: Optional[Mapping[str, struct_pb2.Value]] = None
    ) -> Self:
        """
        Attach information about the internal data source for this change.
        Mutually exclusive with git source.
        """
        self.raw.MergeFrom(
            store_pb2.ChangeDetails(
                internal=store_pb2.ChangeDetails.Internal(
                    source=source, metadata=metadata
                )
            )
        )
        return self


@dataclass
class File:
    path: str
    contents: bytes


@dataclass
class FileOps:
    add: Optional[Iterable[File]] = None
    delete: Optional[Iterable[str]] = None


@dataclass
class FilterPathEqual:
    path: str


@dataclass
class FilterPathContains:
    fragment: str


@dataclass
class FilterPathIn:
    paths: Iterable[str]


@dataclass
class ListFilesResponse:
    raw: store_pb2.ListFilesResponse

    def store_version(self) -> int:
        return self.raw.store_version

    def files(self) -> List[str]:
        return [f for f in self.raw.files]

    def __str__(self):
        return json_format.MessageToJson(self.raw)


@dataclass
class ReplaceFilesResponse:
    raw: store_pb2.ReplaceFilesResponse

    def new_store_version(self) -> int:
        return self.raw.new_store_version

    def ignored_files(self) -> Optional[List[str]]:
        if self.raw.ignored_files:
            return [f for f in self.raw.ignored_files]

        return None

    def __str__(self):
        return json_format.MessageToJson(self.raw)


@dataclass
class ModifyFilesResponse:
    raw: store_pb2.ModifyFilesResponse

    def new_store_version(self) -> int:
        return self.raw.new_store_version

    def __str__(self):
        return json_format.MessageToJson(self.raw)


@dataclass
class GetFilesResponse:
    raw: store_pb2.GetFilesResponse

    def store_version(self) -> int:
        return self.raw.store_version

    def files(self) -> Iterable[File]:
        return (File(path=f.path, contents=f.contents) for f in self.raw.files)

    def files_as_map(self) -> Mapping[str, bytes]:
        return {f.path: f.contents for f in self.raw.files}

    def __str__(self):
        return json_format.MessageToJson(self.raw)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cerbos/schema/v1/schema.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'cerbos/schema/v1/schema.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1d\x63\x65rbos/schema/v1/schema.proto\x12\x10\x63\x65rbos.schema.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\"\xce\x01\n\x0fValidationError\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12@\n\x06source\x18\x03 \x01(\x0e\x32(.cerbos.schema.v1.ValidationError.SourceR\x06source\"K\n\x06Source\x12\x16\n\x12SOURCE_UNSPECIFIED\x10\x00\x12\x14\n\x10SOURCE_PRINCIPAL\x10\x01\x12\x13\n\x0fSOURCE_RESOURCE\x10\x02\"\xcf\x01\n\x06Schema\x12W\n\x02id\x18\x01 \x01(\tBG\x92\x41\x34\x32 Unique identifier for the schemaJ\x10\"principal.json\"\xe0\x41\x02\xbaH\nr\x05\x10\x01\x18\xff\x01\xc8\x01\x01R\x02id\x12l\n\ndefinition\x18\x02 \x01(\x0c\x42L\x92\x41<2\x16JSON schema definitionJ\"{\"type\":\"object\", \"properties\":{}}\xe0\x41\x02\xbaH\x07z\x02\x10\n\xc8\x01\x01R\ndefinitionBo\n\x18\x64\x65v.cerbos.api.v1.schemaZ<github.com/cerbos/cerbos/api/genpb/cerbos/schema/v1;schemav1\xaa\x02\x14\x43\x65rbos.Api.V1.Schemab\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cerbos.schema.v1.schema_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030dev.cerbos.api.v1.schemaZ<github.com/cerbos/cerbos/api/genpb/cerbos/schema/v1;schemav1\252\002\024Cerbos.Api.V1.Schema'
  _globals['_SCHEMA'].fields_by_name['id']._loaded_options = None
  _globals['_SCHEMA'].fields_by_name['id']._serialized_options = b'\222A42 Unique identifier for the schemaJ\020\"principal.json\"\340A\002\272H\nr\005\020\001\030\377\001\310\001\001'
  _globals['_SCHEMA'].fields_by_name['definition']._loaded_options = None
  _globals['_SCHEMA'].fields_by_name['definition']._serialized_options = b'\222A<2\026JSON schema definitionJ\"{\"type\":\"object\", \"properties\":{}}\340A\002\272H\007z\002\020\n\310\001\001'
  _globals['_VALIDATIONERROR']._serialized_start=162
  _globals['_VALIDATIONERROR']._serialized_end=368
  _globals['_VALIDATIONERROR_SOURCE']._serialized_start=293
  _globals['_VALIDATIONERROR_SOURCE']._serialized_end=368
  _globals['_SCHEMA']._serialized_start=371
  _globals['_SCHEMA']._serialized_end=578
# @@protoc_insertion_point(module_scope)



# Copyright 2021-2025 Zenauth Ltd.
# SPDX-License-Identifier: Apache-2.0

import importlib.metadata
import pkgutil

__version__ = importlib.metadata.version(__package__ or __name__)
__path__ = pkgutil.extend_path(__path__, __name__)



from cerbos.policy.v1 import policy_pb2 as _policy_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Metadata(_message.Message):
    __slots__ = ["version", "policies", "build_timestamp", "commit_hash", "source_attributes"]
    class SourceAttributesEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: _policy_pb2.SourceAttributes
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_policy_pb2.SourceAttributes, _Mapping]] = ...) -> None: ...
    VERSION_FIELD_NUMBER: _ClassVar[int]
    POLICIES_FIELD_NUMBER: _ClassVar[int]
    BUILD_TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    COMMIT_HASH_FIELD_NUMBER: _ClassVar[int]
    SOURCE_ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    version: str
    policies: _containers.RepeatedScalarFieldContainer[str]
    build_timestamp: int
    commit_hash: str
    source_attributes: _containers.MessageMap[str, _policy_pb2.SourceAttributes]
    def __init__(self, version: _Optional[str] = ..., policies: _Optional[_Iterable[str]] = ..., build_timestamp: _Optional[int] = ..., commit_hash: _Optional[str] = ..., source_attributes: _Optional[_Mapping[str, _policy_pb2.SourceAttributes]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cerbos/cloud/epdp/v1/epdp.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'cerbos/cloud/epdp/v1/epdp.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cerbos.policy.v1 import policy_pb2 as cerbos_dot_policy_dot_v1_dot_policy__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x63\x65rbos/cloud/epdp/v1/epdp.proto\x12\x14\x63\x65rbos.cloud.epdp.v1\x1a\x1d\x63\x65rbos/policy/v1/policy.proto\"\xda\x02\n\x08Metadata\x12\x18\n\x07version\x18\x01 \x01(\tR\x07version\x12\x1e\n\x08policies\x18\x02 \x03(\tB\x02\x18\x01R\x08policies\x12\'\n\x0f\x62uild_timestamp\x18\x03 \x01(\x03R\x0e\x62uildTimestamp\x12\x1f\n\x0b\x63ommit_hash\x18\x04 \x01(\tR\ncommitHash\x12\x61\n\x11source_attributes\x18\x05 \x03(\x0b\x32\x34.cerbos.cloud.epdp.v1.Metadata.SourceAttributesEntryR\x10sourceAttributes\x1ag\n\x15SourceAttributesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x38\n\x05value\x18\x02 \x01(\x0b\x32\".cerbos.policy.v1.SourceAttributesR\x05value:\x02\x38\x01\x42x\n\x1c\x64\x65v.cerbos.api.cloud.v1.epdpZ=github.com/cerbos/cloud-api/genpb/cerbos/cloud/epdp/v1;epdpv1\xaa\x02\x18\x43\x65rbos.Api.Cloud.V1.Epdpb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cerbos.cloud.epdp.v1.epdp_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034dev.cerbos.api.cloud.v1.epdpZ=github.com/cerbos/cloud-api/genpb/cerbos/cloud/epdp/v1;epdpv1\252\002\030Cerbos.Api.Cloud.V1.Epdp'
  _globals['_METADATA_SOURCEATTRIBUTESENTRY']._loaded_options = None
  _globals['_METADATA_SOURCEATTRIBUTESENTRY']._serialized_options = b'8\001'
  _globals['_METADATA'].fields_by_name['policies']._loaded_options = None
  _globals['_METADATA'].fields_by_name['policies']._serialized_options = b'\030\001'
  _globals['_METADATA']._serialized_start=89
  _globals['_METADATA']._serialized_end=435
  _globals['_METADATA_SOURCEATTRIBUTESENTRY']._serialized_start=332
  _globals['_METADATA_SOURCEATTRIBUTESENTRY']._serialized_end=435
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cerbos/cloud/bundle/v1/bundle.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'cerbos/cloud/bundle/v1/bundle.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from cerbos.cloud.pdp.v1 import pdp_pb2 as cerbos_dot_cloud_dot_pdp_dot_v1_dot_pdp__pb2
from google.api import visibility_pb2 as google_dot_api_dot_visibility__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#cerbos/cloud/bundle/v1/bundle.proto\x12\x16\x63\x65rbos.cloud.bundle.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1d\x63\x65rbos/cloud/pdp/v1/pdp.proto\x1a\x1bgoogle/api/visibility.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xb7\x02\n\nBundleInfo\x12\x1d\n\x05label\x18\x01 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\x05label\x12(\n\x0b\x62undle_hash\x18\x02 \x01(\x0c\x42\x07\xbaH\x04z\x02h R\nbundleHash\x12P\n\x08segments\x18\x03 \x03(\x0b\x32*.cerbos.cloud.bundle.v1.BundleInfo.SegmentB\x08\xbaH\x05\x92\x01\x02\x08\x01R\x08segments\x1a\x8d\x01\n\x07Segment\x12&\n\nsegment_id\x18\x01 \x01(\rB\x07\xbaH\x04*\x02 \x00R\tsegmentId\x12#\n\x08\x63hecksum\x18\x02 \x01(\x0c\x42\x07\xbaH\x04z\x02h R\x08\x63hecksum\x12\x35\n\rdownload_urls\x18\x03 \x03(\tB\x10\xbaH\r\x92\x01\n\x08\x01\x18\x01\"\x04r\x02\x10\x01R\x0c\x64ownloadUrls\">\n\x04Meta\x12\x1e\n\nidentifier\x18\x01 \x01(\tR\nidentifier\x12\x16\n\x06source\x18\x02 \x01(\tR\x06source\"\x8d\x02\n\x08Manifest\x12\x1f\n\x0b\x61pi_version\x18\x01 \x01(\tR\napiVersion\x12T\n\x0cpolicy_index\x18\x02 \x03(\x0b\x32\x31.cerbos.cloud.bundle.v1.Manifest.PolicyIndexEntryR\x0bpolicyIndex\x12\x18\n\x07schemas\x18\x03 \x03(\tR\x07schemas\x12\x30\n\x04meta\x18\x04 \x01(\x0b\x32\x1c.cerbos.cloud.bundle.v1.MetaR\x04meta\x1a>\n\x10PolicyIndexEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"~\n\x10GetBundleRequest\x12>\n\x06pdp_id\x18\x01 \x01(\x0b\x32\x1f.cerbos.cloud.pdp.v1.IdentifierB\x06\xbaH\x03\xc8\x01\x01R\x05pdpId\x12*\n\x0c\x62undle_label\x18\x02 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\x0b\x62undleLabel\"`\n\x11GetBundleResponse\x12K\n\x0b\x62undle_info\x18\x01 \x01(\x0b\x32\".cerbos.cloud.bundle.v1.BundleInfoB\x06\xbaH\x03\xc8\x01\x01R\nbundleInfo\"\xcf\x03\n\x12WatchBundleRequest\x12>\n\x06pdp_id\x18\x01 \x01(\x0b\x32\x1f.cerbos.cloud.pdp.v1.IdentifierB\x06\xbaH\x03\xc8\x01\x01R\x05pdpId\x12X\n\x0bwatch_label\x18\x02 \x01(\x0b\x32\x35.cerbos.cloud.bundle.v1.WatchBundleRequest.WatchLabelH\x00R\nwatchLabel\x12T\n\theartbeat\x18\x03 \x01(\x0b\x32\x34.cerbos.cloud.bundle.v1.WatchBundleRequest.HeartbeatH\x00R\theartbeat\x1a\x38\n\nWatchLabel\x12*\n\x0c\x62undle_label\x18\x01 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\x0b\x62undleLabel\x1a\x87\x01\n\tHeartbeat\x12G\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\r\xbaH\n\xb2\x01\x04J\x02\x08<\xc8\x01\x01R\ttimestamp\x12\x31\n\x10\x61\x63tive_bundle_id\x18\x02 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\x0e\x61\x63tiveBundleIdB\x05\n\x03msg\"\x94\x03\n\x13WatchBundleResponse\x12I\n\rbundle_update\x18\x01 \x01(\x0b\x32\".cerbos.cloud.bundle.v1.BundleInfoH\x00R\x0c\x62undleUpdate\x12U\n\treconnect\x18\x02 \x01(\x0b\x32\x35.cerbos.cloud.bundle.v1.WatchBundleResponse.ReconnectH\x00R\treconnect\x12\x62\n\x0e\x62undle_removed\x18\x03 \x01(\x0b\x32\x39.cerbos.cloud.bundle.v1.WatchBundleResponse.BundleRemovedH\x00R\rbundleRemoved\x1aX\n\tReconnect\x12\x33\n\x07\x62\x61\x63koff\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x07\x62\x61\x63koff\x12\x16\n\x06reason\x18\x02 \x01(\tR\x06reason\x1a\x0f\n\rBundleRemovedB\x0c\n\x03msg\x12\x05\xbaH\x02\x08\x01\x32\xfd\x01\n\x13\x43\x65rbosBundleService\x12\x62\n\tGetBundle\x12(.cerbos.cloud.bundle.v1.GetBundleRequest\x1a).cerbos.cloud.bundle.v1.GetBundleResponse\"\x00\x12l\n\x0bWatchBundle\x12*.cerbos.cloud.bundle.v1.WatchBundleRequest\x1a+.cerbos.cloud.bundle.v1.WatchBundleResponse\"\x00(\x01\x30\x01\x1a\x14\xfa\xd2\xe4\x93\x02\x0e\x12\x0c\x45XPERIMENTALB\x80\x01\n\x1e\x64\x65v.cerbos.api.cloud.v1.bundleZAgithub.com/cerbos/cloud-api/genpb/cerbos/cloud/bundle/v1;bundlev1\xaa\x02\x1a\x43\x65rbos.Api.Cloud.V1.Bundleb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cerbos.cloud.bundle.v1.bundle_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\036dev.cerbos.api.cloud.v1.bundleZAgithub.com/cerbos/cloud-api/genpb/cerbos/cloud/bundle/v1;bundlev1\252\002\032Cerbos.Api.Cloud.V1.Bundle'
  _globals['_BUNDLEINFO_SEGMENT'].fields_by_name['segment_id']._loaded_options = None
  _globals['_BUNDLEINFO_SEGMENT'].fields_by_name['segment_id']._serialized_options = b'\272H\004*\002 \000'
  _globals['_BUNDLEINFO_SEGMENT'].fields_by_name['checksum']._loaded_options = None
  _globals['_BUNDLEINFO_SEGMENT'].fields_by_name['checksum']._serialized_options = b'\272H\004z\002h '
  _globals['_BUNDLEINFO_SEGMENT'].fields_by_name['download_urls']._loaded_options = None
  _globals['_BUNDLEINFO_SEGMENT'].fields_by_name['download_urls']._serialized_options = b'\272H\r\222\001\n\010\001\030\001\"\004r\002\020\001'
  _globals['_BUNDLEINFO'].fields_by_name['label']._loaded_options = None
  _globals['_BUNDLEINFO'].fields_by_name['label']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_BUNDLEINFO'].fields_by_name['bundle_hash']._loaded_options = None
  _globals['_BUNDLEINFO'].fields_by_name['bundle_hash']._serialized_options = b'\272H\004z\002h '
  _globals['_BUNDLEINFO'].fields_by_name['segments']._loaded_options = None
  _globals['_BUNDLEINFO'].fields_by_name['segments']._serialized_options = b'\272H\005\222\001\002\010\001'
  _globals['_MANIFEST_POLICYINDEXENTRY']._loaded_options = None
  _globals['_MANIFEST_POLICYINDEXENTRY']._serialized_options = b'8\001'
  _globals['_GETBUNDLEREQUEST'].fields_by_name['pdp_id']._loaded_options = None
  _globals['_GETBUNDLEREQUEST'].fields_by_name['pdp_id']._serialized_options = b'\272H\003\310\001\001'
  _globals['_GETBUNDLEREQUEST'].fields_by_name['bundle_label']._loaded_options = None
  _globals['_GETBUNDLEREQUEST'].fields_by_name['bundle_label']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_GETBUNDLERESPONSE'].fields_by_name['bundle_info']._loaded_options = None
  _globals['_GETBUNDLERESPONSE'].fields_by_name['bundle_info']._serialized_options = b'\272H\003\310\001\001'
  _globals['_WATCHBUNDLEREQUEST_WATCHLABEL'].fields_by_name['bundle_label']._loaded_options = None
  _globals['_WATCHBUNDLEREQUEST_WATCHLABEL'].fields_by_name['bundle_label']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_WATCHBUNDLEREQUEST_HEARTBEAT'].fields_by_name['timestamp']._loaded_options = None
  _globals['_WATCHBUNDLEREQUEST_HEARTBEAT'].fields_by_name['timestamp']._serialized_options = b'\272H\n\262\001\004J\002\010<\310\001\001'
  _globals['_WATCHBUNDLEREQUEST_HEARTBEAT'].fields_by_name['active_bundle_id']._loaded_options = None
  _globals['_WATCHBUNDLEREQUEST_HEARTBEAT'].fields_by_name['active_bundle_id']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_WATCHBUNDLEREQUEST'].fields_by_name['pdp_id']._loaded_options = None
  _globals['_WATCHBUNDLEREQUEST'].fields_by_name['pdp_id']._serialized_options = b'\272H\003\310\001\001'
  _globals['_WATCHBUNDLERESPONSE'].oneofs_by_name['msg']._loaded_options = None
  _globals['_WATCHBUNDLERESPONSE'].oneofs_by_name['msg']._serialized_options = b'\272H\002\010\001'
  _globals['_CERBOSBUNDLESERVICE']._loaded_options = None
  _globals['_CERBOSBUNDLESERVICE']._serialized_options = b'\372\322\344\223\002\016\022\014EXPERIMENTAL'
  _globals['_BUNDLEINFO']._serialized_start=218
  _globals['_BUNDLEINFO']._serialized_end=529
  _globals['_BUNDLEINFO_SEGMENT']._serialized_start=388
  _globals['_BUNDLEINFO_SEGMENT']._serialized_end=529
  _globals['_META']._serialized_start=531
  _globals['_META']._serialized_end=593
  _globals['_MANIFEST']._serialized_start=596
  _globals['_MANIFEST']._serialized_end=865
  _globals['_MANIFEST_POLICYINDEXENTRY']._serialized_start=803
  _globals['_MANIFEST_POLICYINDEXENTRY']._serialized_end=865
  _globals['_GETBUNDLEREQUEST']._serialized_start=867
  _globals['_GETBUNDLEREQUEST']._serialized_end=993
  _globals['_GETBUNDLERESPONSE']._serialized_start=995
  _globals['_GETBUNDLERESPONSE']._serialized_end=1091
  _globals['_WATCHBUNDLEREQUEST']._serialized_start=1094
  _globals['_WATCHBUNDLEREQUEST']._serialized_end=1557
  _globals['_WATCHBUNDLEREQUEST_WATCHLABEL']._serialized_start=1356
  _globals['_WATCHBUNDLEREQUEST_WATCHLABEL']._serialized_end=1412
  _globals['_WATCHBUNDLEREQUEST_HEARTBEAT']._serialized_start=1415
  _globals['_WATCHBUNDLEREQUEST_HEARTBEAT']._serialized_end=1550
  _globals['_WATCHBUNDLERESPONSE']._serialized_start=1560
  _globals['_WATCHBUNDLERESPONSE']._serialized_end=1964
  _globals['_WATCHBUNDLERESPONSE_RECONNECT']._serialized_start=1845
  _globals['_WATCHBUNDLERESPONSE_RECONNECT']._serialized_end=1933
  _globals['_WATCHBUNDLERESPONSE_BUNDLEREMOVED']._serialized_start=1935
  _globals['_WATCHBUNDLERESPONSE_BUNDLEREMOVED']._serialized_end=1950
  _globals['_CERBOSBUNDLESERVICE']._serialized_start=1967
  _globals['_CERBOSBUNDLESERVICE']._serialized_end=2220
# @@protoc_insertion_point(module_scope)



from buf.validate import validate_pb2 as _validate_pb2
from google.api import visibility_pb2 as _visibility_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class IssueAccessTokenRequest(_message.Message):
    __slots__ = ["client_id", "client_secret"]
    CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
    CLIENT_SECRET_FIELD_NUMBER: _ClassVar[int]
    client_id: str
    client_secret: str
    def __init__(self, client_id: _Optional[str] = ..., client_secret: _Optional[str] = ...) -> None: ...

class IssueAccessTokenResponse(_message.Message):
    __slots__ = ["access_token", "expires_in"]
    ACCESS_TOKEN_FIELD_NUMBER: _ClassVar[int]
    EXPIRES_IN_FIELD_NUMBER: _ClassVar[int]
    access_token: str
    expires_in: _duration_pb2.Duration
    def __init__(self, access_token: _Optional[str] = ..., expires_in: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cerbos/cloud/logs/v1/logs.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'cerbos/cloud/logs/v1/logs.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from cerbos.audit.v1 import audit_pb2 as cerbos_dot_audit_dot_v1_dot_audit__pb2
from cerbos.cloud.pdp.v1 import pdp_pb2 as cerbos_dot_cloud_dot_pdp_dot_v1_dot_pdp__pb2
from google.api import visibility_pb2 as google_dot_api_dot_visibility__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x63\x65rbos/cloud/logs/v1/logs.proto\x12\x14\x63\x65rbos.cloud.logs.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1b\x63\x65rbos/audit/v1/audit.proto\x1a\x1d\x63\x65rbos/cloud/pdp/v1/pdp.proto\x1a\x1bgoogle/api/visibility.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x97\x04\n\x0bIngestBatch\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12N\n\x07\x65ntries\x18\x02 \x03(\x0b\x32\'.cerbos.cloud.logs.v1.IngestBatch.EntryB\x0b\xbaH\x08\x92\x01\x05\x08\x01\x10\x80\x08R\x07\x65ntries\x1a\xc6\x02\n\x05\x45ntry\x12K\n\x04kind\x18\x01 \x01(\x0e\x32+.cerbos.cloud.logs.v1.IngestBatch.EntryKindB\n\xbaH\x07\x82\x01\x04\x18\x01\x18\x02R\x04kind\x12@\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x06\xbaH\x03\xc8\x01\x01R\ttimestamp\x12K\n\x10\x61\x63\x63\x65ss_log_entry\x18\x03 \x01(\x0b\x32\x1f.cerbos.audit.v1.AccessLogEntryH\x00R\x0e\x61\x63\x63\x65ssLogEntry\x12Q\n\x12\x64\x65\x63ision_log_entry\x18\x04 \x01(\x0b\x32!.cerbos.audit.v1.DecisionLogEntryH\x00R\x10\x64\x65\x63isionLogEntryB\x0e\n\x05\x65ntry\x12\x05\xbaH\x02\x08\x01\"_\n\tEntryKind\x12\x1a\n\x16\x45NTRY_KIND_UNSPECIFIED\x10\x00\x12\x19\n\x15\x45NTRY_KIND_ACCESS_LOG\x10\x01\x12\x1b\n\x17\x45NTRY_KIND_DECISION_LOG\x10\x02\"\x90\x01\n\rIngestRequest\x12>\n\x06pdp_id\x18\x01 \x01(\x0b\x32\x1f.cerbos.cloud.pdp.v1.IdentifierB\x06\xbaH\x03\xc8\x01\x01R\x05pdpId\x12?\n\x05\x62\x61tch\x18\x02 \x01(\x0b\x32!.cerbos.cloud.logs.v1.IngestBatchB\x06\xbaH\x03\xc8\x01\x01R\x05\x62\x61tch\"\xda\x01\n\x0eIngestResponse\x12\x32\n\x07success\x18\x01 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00R\x07success\x12H\n\x07\x62\x61\x63koff\x18\x02 \x01(\x0b\x32,.cerbos.cloud.logs.v1.IngestResponse.BackoffH\x00R\x07\x62\x61\x63koff\x1a@\n\x07\x42\x61\x63koff\x12\x35\n\x08\x64uration\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x08\x64urationB\x08\n\x06status2\x80\x01\n\x11\x43\x65rbosLogsService\x12U\n\x06Ingest\x12#.cerbos.cloud.logs.v1.IngestRequest\x1a$.cerbos.cloud.logs.v1.IngestResponse\"\x00\x1a\x14\xfa\xd2\xe4\x93\x02\x0e\x12\x0c\x45XPERIMENTALBx\n\x1c\x64\x65v.cerbos.api.cloud.v1.logsZ=github.com/cerbos/cloud-api/genpb/cerbos/cloud/logs/v1;logsv1\xaa\x02\x18\x43\x65rbos.Api.Cloud.V1.Logsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cerbos.cloud.logs.v1.logs_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034dev.cerbos.api.cloud.v1.logsZ=github.com/cerbos/cloud-api/genpb/cerbos/cloud/logs/v1;logsv1\252\002\030Cerbos.Api.Cloud.V1.Logs'
  _globals['_INGESTBATCH_ENTRY'].oneofs_by_name['entry']._loaded_options = None
  _globals['_INGESTBATCH_ENTRY'].oneofs_by_name['entry']._serialized_options = b'\272H\002\010\001'
  _globals['_INGESTBATCH_ENTRY'].fields_by_name['kind']._loaded_options = None
  _globals['_INGESTBATCH_ENTRY'].fields_by_name['kind']._serialized_options = b'\272H\007\202\001\004\030\001\030\002'
  _globals['_INGESTBATCH_ENTRY'].fields_by_name['timestamp']._loaded_options = None
  _globals['_INGESTBATCH_ENTRY'].fields_by_name['timestamp']._serialized_options = b'\272H\003\310\001\001'
  _globals['_INGESTBATCH'].fields_by_name['entries']._loaded_options = None
  _globals['_INGESTBATCH'].fields_by_name['entries']._serialized_options = b'\272H\010\222\001\005\010\001\020\200\010'
  _globals['_INGESTREQUEST'].fields_by_name['pdp_id']._loaded_options = None
  _globals['_INGESTREQUEST'].fields_by_name['pdp_id']._serialized_options = b'\272H\003\310\001\001'
  _globals['_INGESTREQUEST'].fields_by_name['batch']._loaded_options = None
  _globals['_INGESTREQUEST'].fields_by_name['batch']._serialized_options = b'\272H\003\310\001\001'
  _globals['_CERBOSLOGSSERVICE']._loaded_options = None
  _globals['_CERBOSLOGSSERVICE']._serialized_options = b'\372\322\344\223\002\016\022\014EXPERIMENTAL'
  _globals['_INGESTBATCH']._serialized_start=270
  _globals['_INGESTBATCH']._serialized_end=805
  _globals['_INGESTBATCH_ENTRY']._serialized_start=382
  _globals['_INGESTBATCH_ENTRY']._serialized_end=708
  _globals['_INGESTBATCH_ENTRYKIND']._serialized_start=710
  _globals['_INGESTBATCH_ENTRYKIND']._serialized_end=805
  _globals['_INGESTREQUEST']._serialized_start=808
  _globals['_INGESTREQUEST']._serialized_end=952
  _globals['_INGESTRESPONSE']._serialized_start=955
  _globals['_INGESTRESPONSE']._serialized_end=1173
  _globals['_INGESTRESPONSE_BACKOFF']._serialized_start=1099
  _globals['_INGESTRESPONSE_BACKOFF']._serialized_end=1163
  _globals['_CERBOSLOGSSERVICE']._serialized_start=1176
  _globals['_CERBOSLOGSSERVICE']._serialized_end=1304
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from cerbos.cloud.apikey.v1 import apikey_pb2 as cerbos_dot_cloud_dot_apikey_dot_v1_dot_apikey__pb2


class ApiKeyServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.IssueAccessToken = channel.unary_unary(
                '/cerbos.cloud.apikey.v1.ApiKeyService/IssueAccessToken',
                request_serializer=cerbos_dot_cloud_dot_apikey_dot_v1_dot_apikey__pb2.IssueAccessTokenRequest.SerializeToString,
                response_deserializer=cerbos_dot_cloud_dot_apikey_dot_v1_dot_apikey__pb2.IssueAccessTokenResponse.FromString,
                _registered_method=True)


class ApiKeyServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def IssueAccessToken(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ApiKeyServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'IssueAccessToken': grpc.unary_unary_rpc_method_handler(
                    servicer.IssueAccessToken,
                    request_deserializer=cerbos_dot_cloud_dot_apikey_dot_v1_dot_apikey__pb2.IssueAccessTokenRequest.FromString,
                    response_serializer=cerbos_dot_cloud_dot_apikey_dot_v1_dot_apikey__pb2.IssueAccessTokenResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cerbos.cloud.apikey.v1.ApiKeyService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('cerbos.cloud.apikey.v1.ApiKeyService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class ApiKeyService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def IssueAccessToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/cerbos.cloud.apikey.v1.ApiKeyService/IssueAccessToken',
            cerbos_dot_cloud_dot_apikey_dot_v1_dot_apikey__pb2.IssueAccessTokenRequest.SerializeToString,
            cerbos_dot_cloud_dot_apikey_dot_v1_dot_apikey__pb2.IssueAccessTokenResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



# Copyright 2021-2025 Zenauth Ltd.
# SPDX-License-Identifier: Apache-2.0
import os
from pathlib import Path
from typing import Any, List, Optional

import pytest

from cerbos.sdk.hub import util
from cerbos.sdk.hub.store import CerbosHubStoreClient
from cerbos.sdk.hub.store_model import (
    ChangeDetails,
    ConditionUnsatisfiedError,
    FileOps,
    FilterPathEqual,
    FilterPathIn,
    InvalidRequestError,
    OperationDiscardedError,
    ValidationFailureError,
)

pytestmark = pytest.mark.anyio


@pytest.fixture(scope="module")
def hub_store_client(anyio_backend):
    with CerbosHubStoreClient() as client:
        yield client


@pytest.fixture(scope="module")
def store_id():
    return os.getenv("CERBOS_HUB_STORE_ID")


@pytest.fixture(scope="module")
def testdata_dir(pytestconfig):
    return Path(pytestconfig.rootdir, "tests", "_hub", "testdata")


@pytest.fixture(scope="module")
def zip_data(testdata_dir):
    return util.zip_directory(Path(testdata_dir, "replace_files", "success"))


@pytest.fixture(scope="class")
def reset_store(hub_store_client, store_id, zip_data):
    hub_store_client.replace_files_lenient(store_id, "Test replace", zip_data)


_WANT_FILES = [
    "_schemas/principal.json",
    "_schemas/resources/leave_request.json",
    "_schemas/resources/purchase_order.json",
    "_schemas/resources/salary_record.json",
    "derived_roles/common_roles.yaml",
    "derived_roles/derived_roles_01.yaml",
    "derived_roles/derived_roles_02.yaml",
    "derived_roles/derived_roles_03.yaml",
    "derived_roles/derived_roles_04.yaml",
    "derived_roles/derived_roles_05.yaml",
    "export_constants/export_constants_01.yaml",
    "export_variables/export_variables_01.yaml",
    "principal_policies/policy_01.yaml",
    "principal_policies/policy_02.yaml",
    "principal_policies/policy_02_acme.hr.yaml",
    "principal_policies/policy_02_acme.sales.yaml",
    "principal_policies/policy_02_acme.yaml",
    "principal_policies/policy_03.yaml",
    "principal_policies/policy_04.yaml",
    "principal_policies/policy_05.yaml",
    "principal_policies/policy_06.yaml",
    "resource_policies/disabled_policy_01.yaml",
    "resource_policies/policy_01.yaml",
    "resource_policies/policy_02.yaml",
    "resource_policies/policy_03.yaml",
    "resource_policies/policy_04.yaml",
    "resource_policies/policy_04_test.yaml",
    "resource_policies/policy_05.yaml",
    "resource_policies/policy_05_acme.hr.uk.brighton.kemptown.yaml",
    "resource_policies/policy_05_acme.hr.uk.brighton.yaml",
    "resource_policies/policy_05_acme.hr.uk.london.yaml",
    "resource_policies/policy_05_acme.hr.uk.yaml",
    "resource_policies/policy_05_acme.hr.yaml",
    "resource_policies/policy_05_acme.yaml",
    "resource_policies/policy_06.yaml",
    "resource_policies/policy_07.yaml",
    "resource_policies/policy_07_acme.yaml",
    "resource_policies/policy_08.yaml",
    "resource_policies/policy_09.yaml",
    "resource_policies/policy_10.yaml",
    "resource_policies/policy_11.yaml",
    "resource_policies/policy_12.yaml",
    "resource_policies/policy_13.yaml",
    "resource_policies/policy_14.yaml",
    "resource_policies/policy_15.yaml",
    "resource_policies/policy_16.yaml",
    "resource_policies/policy_17.acme.sales.yaml",
    "resource_policies/policy_17.acme.yaml",
    "resource_policies/policy_17.yaml",
    "resource_policies/policy_18.yaml",
    "role_policies/policy_01_acme.hr.uk.brighton.yaml",
    "role_policies/policy_02_acme.hr.uk.brighton.yaml",
    "role_policies/policy_03_acme.hr.uk.yaml",
    "role_policies/policy_04_acme.hr.uk.yaml",
    "role_policies/policy_05_acme.hr.uk.london.yaml",
    "role_policies/policy_06_acme.hr.uk.brighton.kemptown.yaml",
    "tests/policy_04_test.yaml",
    "tests/policy_05_test.yaml",
]


@pytest.mark.skipif(
    os.getenv("CERBOS_HUB_CLIENT_ID") == ""
    or os.getenv("CERBOS_HUB_CLIENT_SECRET") == ""
    or os.getenv("CERBOS_HUB_STORE_ID") == "",
    reason="Cerbos Hub credentials not defined",
)
class TestCerbosHubStoreClient:
    class TestReplaceFiles:
        def test_operation_discarded_with_zip(
            self,
            store_id: str,
            testdata_dir: Path,
            hub_store_client: CerbosHubStoreClient,
            reset_store: Any,
        ):
            with pytest.raises(OperationDiscardedError):
                contents = util.zip_directory(
                    Path(testdata_dir, "replace_files", "success")
                )
                hub_store_client.replace_files(store_id, "Test replace", contents)
            assert_store_contents(hub_store_client, store_id)

        def test_operation_discarded_with_files(
            self,
            store_id: str,
            testdata_dir: Path,
            hub_store_client: CerbosHubStoreClient,
            reset_store: Any,
        ):
            with pytest.raises(OperationDiscardedError):
                hub_store_client.replace_files(
                    store_id,
                    "Test replace files",
                    util.iter_files(Path(testdata_dir, "replace_files", "success")),
                )
            assert_store_contents(hub_store_client, store_id)

        def test_invalid_files(
            self,
            store_id: str,
            testdata_dir: Path,
            hub_store_client: CerbosHubStoreClient,
            reset_store: Any,
        ):
            with pytest.raises(ValidationFailureError) as validation_error:
                contents = util.zip_directory(
                    Path(testdata_dir, "replace_files", "invalid")
                )
                hub_store_client.replace_files(store_id, "Test replace", contents)
            assert (
                isinstance(validation_error.value, ValidationFailureError)
                and validation_error.value.errors is not None
            )
            assert_store_contents(hub_store_client, store_id)

        def test_unsuccessful_condition(
            self,
            store_id: str,
            testdata_dir: Path,
            hub_store_client: CerbosHubStoreClient,
            reset_store: Any,
        ):
            with pytest.raises(ConditionUnsatisfiedError) as unsatisfied_error:
                contents = util.zip_directory(
                    Path(testdata_dir, "replace_files", "conditional")
                )
                hub_store_client.replace_files(
                    store_id,
                    "Test replace",
                    contents,
                    version_must_equal=999999999999999,
                )
            assert (
                isinstance(unsatisfied_error.value, ConditionUnsatisfiedError)
                and unsatisfied_error.value.current_store_version is not None
            )
            assert_store_contents(hub_store_client, store_id)

    class TestModifyFiles:
        def test_success(
            self,
            store_id: str,
            testdata_dir: Path,
            hub_store_client: CerbosHubStoreClient,
            reset_store: Any,
        ):
            haveAdd = hub_store_client.modify_files(
                store_id,
                "Test modify",
                FileOps(
                    add=util.iter_files(Path(testdata_dir, "modify_files", "success"))
                ),
                change_details=ChangeDetails("Test modify").with_git_source(
                    "cerbos/foo", "x"
                ),
            )
            assert haveAdd.new_store_version() > 0
            assert_store_contents(hub_store_client, store_id, ["example.yaml"])
            haveFiles = hub_store_client.get_files(store_id, ["example.yaml"])
            haveFileContents = haveFiles.files_as_map()["example.yaml"]
            assert (
                Path(
                    testdata_dir, "modify_files", "success", "example.yaml"
                ).read_bytes()
                == haveFileContents
            )
            haveDelete = hub_store_client.modify_files(
                store_id, "Test delete", FileOps(delete=["example.yaml"])
            )
            assert haveDelete.new_store_version() > haveAdd.new_store_version()
            assert_store_contents(hub_store_client, store_id)

        def test_invalid_request(
            self,
            store_id: str,
            testdata_dir: Path,
            hub_store_client: CerbosHubStoreClient,
            reset_store: Any,
        ):
            with pytest.raises(InvalidRequestError):
                hub_store_client.modify_files(store_id, "Test modify", FileOps())

        def test_invalid_files(
            self,
            store_id: str,
            testdata_dir: Path,
            hub_store_client: CerbosHubStoreClient,
            reset_store: Any,
        ):
            with pytest.raises(ValidationFailureError) as validation_error:
                hub_store_client.modify_files(
                    store_id,
                    "Test modify",
                    FileOps(
                        add=util.iter_files(
                            Path(testdata_dir, "modify_files", "invalid")
                        )
                    ),
                )
            assert (
                isinstance(validation_error.value, ValidationFailureError)
                and validation_error.value.errors is not None
            )

        def test_unsatisfied_condition(
            self,
            store_id: str,
            testdata_dir: Path,
            hub_store_client: CerbosHubStoreClient,
            reset_store: Any,
        ):
            with pytest.raises(ConditionUnsatisfiedError):
                hub_store_client.modify_files(
                    store_id,
                    "Test modify",
                    FileOps(
                        add=util.iter_files(
                            Path(testdata_dir, "modify_files", "conditional")
                        )
                    ),
                    version_must_equal=99999999999,
                )

    class TestGetFiles:
        def test_success(
            self,
            store_id: str,
            hub_store_client: CerbosHubStoreClient,
            reset_store: Any,
        ):
            have = hub_store_client.get_files(
                store_id, ["export_constants/export_constants_01.yaml"]
            )
            haveFiles = have.files_as_map()
            assert len(haveFiles) == 1
            assert "export_constants/export_constants_01.yaml" in haveFiles

        def test_non_existent(
            self,
            store_id: str,
            hub_store_client: CerbosHubStoreClient,
            reset_store: Any,
        ):
            have = hub_store_client.get_files(store_id, ["foobar"])
            assert len(have.files_as_map()) == 0

    class TestListFiles:
        def test_filter_match(
            self,
            store_id: str,
            hub_store_client: CerbosHubStoreClient,
            reset_store: Any,
        ):
            have = hub_store_client.list_files(
                store_id,
                FilterPathIn(
                    [
                        "export_constants/export_constants_01.yaml",
                        "export_variables/export_variables_01.yaml",
                    ]
                ),
            )
            haveFiles = have.files()
            assert haveFiles == [
                "export_constants/export_constants_01.yaml",
                "export_variables/export_variables_01.yaml",
            ]

        def test_no_filter_match(
            self,
            store_id: str,
            hub_store_client: CerbosHubStoreClient,
            reset_store: Any,
        ):
            have = hub_store_client.list_files(store_id, FilterPathEqual("foobar"))
            haveFiles = have.files()
            assert len(haveFiles) == 0


def assert_store_contents(
    hub_store_client: CerbosHubStoreClient,
    store_id: str,
    additional: Optional[List[str]] = None,
):
    have = hub_store_client.list_files(store_id)
    assert len(have.files()) > 0
    want = _WANT_FILES.copy()
    if additional:
        want += additional
    assert sorted(have.files()) == sorted(want)
